#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test LCSIM module."""

from __future__ import absolute_import
import unittest
from mock import patch, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Applications import LCSIM
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertMockCalls

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.LCSIM'

# pylint: disable=protected-access,too-many-public-methods


class LCSIMTestCase(unittest.TestCase):
  """Base class for the LCSIM test cases."""

  def setUp(self):
    """set up the objects."""
    self.lcs = LCSIM()

  def test_setoutputrecfile(self):
    self.assertFalse(self.lcs._errorDict)
    self.lcs.setOutputRecFile('myTestRECfile.rec', 'test/path/rec')
    self.assertFalse(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.outputRecFile, 'myTestRECfile.rec', self)
    assertEqualsImproved(self.lcs.outputRecPath, 'test/path/rec', self)

  def test_setoutputrecfile_argcheck_fails(self):
    self.assertFalse(self.lcs._errorDict)
    self.lcs.setOutputRecFile(123)
    self.assertTrue(self.lcs._errorDict)

  def test_setoutputdstfile(self):
    self.assertFalse(self.lcs._errorDict)
    self.lcs.setOutputDstFile('myTestDSTfile.dst', 'test/path/dst')
    self.assertFalse(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.outputDstFile, 'myTestDSTfile.dst', self)
    assertEqualsImproved(self.lcs.outputDstPath, 'test/path/dst', self)

  def test_setoutputdstfile_argcheck_fails(self):
    self.assertFalse(self.lcs._errorDict)
    self.lcs.setOutputDstFile(123)
    self.assertTrue(self.lcs._errorDict)

  def test_setaliasproperties(self):
    with patch('os.path.exists', new=Mock(return_value=False)):
      self.lcs.setAliasProperties('myAlias.Properties.Test')
    assertEqualsImproved(self.lcs.aliasProperties, 'myAlias.Properties.Test', self)
    self.assertFalse(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.inputSB, [], self)

  def test_setaliasproperties_argcheck_fails(self):
    with patch('os.path.exists', new=Mock(return_value=True)):
      self.lcs.setAliasProperties(8934)
    assertEqualsImproved(self.lcs.aliasProperties, 8934, self)
    self.assertTrue(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.inputSB, [8934], self)

  def test_setaliasproperties_lfn(self):
    with patch('os.path.exists', new=Mock(return_value=False)):
      self.lcs.setAliasProperties('LFN:/myAlias.Properties.Test')
    assertEqualsImproved(self.lcs.aliasProperties, 'LFN:/myAlias.Properties.Test', self)
    self.assertFalse(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.inputSB, ['LFN:/myAlias.Properties.Test'], self)

  def test_setdetectormodel(self):
    with patch('os.path.exists', new=Mock(return_value=False)):
      self.lcs.setDetectorModel('Test_Detectorv103.clic')
    assertEqualsImproved(self.lcs.detectorModel, 'Test_Detectorv103.clic', self)
    self.assertFalse(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.inputSB, [], self)

  def test_setdetectormodel_argcheck_fails(self):
    with patch('os.path.exists', new=Mock(return_value=True)):
      self.lcs.setDetectorModel(2489)
    assertEqualsImproved(self.lcs.detectorModel, 2489, self)
    self.assertTrue(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.inputSB, [2489], self)

  def test_setdetectormodel_lfn(self):
    with patch('os.path.exists', new=Mock(return_value=False)):
      self.lcs.setDetectorModel('LFN:/dir/other_dir/DetectorTestmeModel.ilc')
    assertEqualsImproved(self.lcs.detectorModel, 'LFN:/dir/other_dir/DetectorTestmeModel.ilc', self)
    self.assertFalse(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.inputSB, ['LFN:/dir/other_dir/DetectorTestmeModel.ilc'], self)

  def test_settrackingstrategy(self):
    with patch('os.path.exists', new=Mock(return_value=False)):
      self.lcs.setTrackingStrategy('Test_Strategy.trackme')
    assertEqualsImproved(self.lcs.trackingStrategy, 'Test_Strategy.trackme', self)
    self.assertFalse(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.inputSB, [], self)

  def test_settrackingstrategy_argcheck_fails(self):
    with patch('os.path.exists', new=Mock(return_value=True)):
      self.lcs.setTrackingStrategy(4812)
    assertEqualsImproved(self.lcs.trackingStrategy, 4812, self)
    self.assertTrue(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.inputSB, [4812], self)

  def test_settrackingstrategy_lfn(self):
    with patch('os.path.exists', new=Mock(return_value=False)):
      self.lcs.setTrackingStrategy('LFN:/my/track/strat.txt')
    assertEqualsImproved(self.lcs.trackingStrategy, 'LFN:/my/track/strat.txt', self)
    self.assertFalse(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.inputSB, ['LFN:/my/track/strat.txt'], self)

  def test_setextraparams(self):
    self.assertFalse(self.lcs.willBeCut)
    self.assertFalse(self.lcs._errorDict)
    self.lcs.setExtraParams('myTestPar')
    self.lcs.willRunSLICPandora()
    self.assertFalse(self.lcs._errorDict)
    assertEqualsImproved(self.lcs.extraParams, 'myTestPar', self)
    self.assertTrue(self.lcs.willBeCut)

  def test_setextraparams_argcheck_fails(self):
    self.assertFalse(self.lcs._errorDict)
    self.lcs.setExtraParams([8914])
    self.assertTrue(self.lcs._errorDict)

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.lcs._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.lcs._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
        patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.lcs._userjobmodules(None),
                            'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_OK('something'))), \
        patch('%s._setOutputComputeDataList' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_other_test_err'))):
      assertDiracFailsWith(self.lcs._prodjobmodules(None),
                            'prodjobmodules failed', self)

  def test_checkwfconsistency(self):
    assertDiracSucceeds(self.lcs._checkWorkflowConsistency(), self)

  def test_resolveparameters(self):
    step_mock = Mock()
    assertDiracSucceeds(self.lcs._resolveLinkedStepParameters(step_mock()), self)

  def test_resolveparameters_setlink(self):
    step_mock = Mock()
    self.lcs._linkedidx = 1
    self.lcs._jobsteps = [None, Mock()]
    assertDiracSucceeds(self.lcs._resolveLinkedStepParameters(step_mock()), self)

  def test_checkconsistency(self):
    import inspect
    self.lcs.version = 'v2.4'
    self.lcs.steeringFile = 'myTestSteerFile.txt'
    self.lcs.trackingStrategy = 'myTestStrat'
    self.lcs.detectorModel = 'myDetModel.wrongFormat'
    with patch('os.path.exists', new=Mock(side_effect=[False, True, False])) as exists_mock, \
         patch.object(inspect.getmodule(LCSIM), 'Exists', new=Mock(return_value=S_OK())), \
         patch.object(inspect.getmodule(LCSIM), 'checkXMLValidity', new=Mock(return_value=S_OK())) as xml_mock:
      assertDiracFailsWith(self.lcs._checkConsistency('myTestJob'),
                            'you have to pass an existing .zip file', self)
      assertMockCalls(exists_mock, ['myTestSteerFile.txt', 'myTestSteerFile.txt', 'myTestStrat'], self)
      xml_mock.assert_called_once_with('myTestSteerFile.txt')

  def test_checkconsistency_noversion(self):
    assertDiracFailsWith(self.lcs._checkConsistency(), 'no version found', self)

  def test_checkconsistency_basic(self):
    self.lcs.energy = 2489
    self.lcs.numberOfEvents = 8245
    self.lcs.version = 'v1.2'
    self.lcs.steeringFile = ''
    self.lcs.detectorModel = ''
    self.lcs._jobtype = 'User'
    assertDiracSucceeds(self.lcs._checkConsistency(), self)

  def test_checkconsistency_steering_Exists_fails(self):
    import inspect
    self.lcs.version = 'v2.4'
    self.lcs.steeringFile = 'myTestSteerFile.txt'
    self.lcs.trackingStrategy = 'myTestStrat'
    self.lcs.detectorModel = 'myDetModel.wrongFormat'
    with patch('os.path.exists', new=Mock(side_effect=[False])), \
         patch.object(inspect.getmodule(LCSIM), 'Exists', new=Mock(return_value=S_ERROR('Exists_test_err'))):
      assertDiracFailsWith(self.lcs._checkConsistency('myTestJob'), 'exists_test_err', self)

  def test_checkconsistency_invalidxml(self):
    import inspect
    self.lcs.version = 'v2.4'
    self.lcs.steeringFile = 'myTestSteerFile.txt'
    self.lcs.trackingStrategy = 'myTestStrat'
    self.lcs.detectorModel = 'myDetModel.wrongFormat'
    with patch('os.path.exists', new=Mock(side_effect=[True, True])), \
         patch.object(inspect.getmodule(LCSIM), 'checkXMLValidity', new=Mock(return_value=S_ERROR('xmlcheck_failed_testme'))):
      assertDiracFailsWith(self.lcs._checkConsistency('myTestJob'), 'supplied steering file cannot be'
                            ' read by xml parser: xmlcheck_failed_testme', self)

  def test_checkconsistency_tracking_Exists_fails(self):
    import inspect
    self.lcs.version = 'v2.4'
    self.lcs.steeringFile = 'myTestSteerFile.txt'
    self.lcs.trackingStrategy = 'myTestStrat'
    self.lcs.detectorModel = 'myDetModel.wrongFormat'
    with patch('os.path.exists', new=Mock(side_effect=[True, False, False])), \
         patch.object(inspect.getmodule(LCSIM), 'Exists', new=Mock(return_value=S_ERROR('xmlcheck_failed_testme'))):
      assertDiracFailsWith(self.lcs._checkConsistency('myTestJob'), 'xmlcheck_failed_testme', self)

  def test_checkconsistency_othercase(self):
    import inspect
    self.lcs.version = 'v2.4'
    self.lcs.steeringFile = ''
    self.lcs.trackingStrategy = 'lfn:/myTestStrat'
    self.lcs.detectorModel = 'correctDetector.zip'
    self.lcs._jobtype = 'notUser'
    self.lcs._listofoutput = []
    app1 = Mock()
    app1.appame = 'myTestApp'
    app2 = Mock()
    app2.appname = 'marlin'
    self.lcs._inputapp = [app1, app2]
    self.lcs.outputFile = ''
    self.lcs.willBeCut = False
    with patch('os.path.exists', new=Mock(side_effect=[False])), \
         patch.object(inspect.getmodule(LCSIM), 'Exists', new=Mock(return_value=S_OK())):
      assertDiracSucceeds(self.lcs._checkConsistency('myTestJob'), self)
      expected_output_list = [{"outputFile": "@{outputREC}", "outputPath": "@{outputPathREC}",
                                  "outputDataSE": "@{OutputSE}"}, {"outputFile": "@{outputDST}",
                                                                      "outputPath": "@{outputPathDST}",
                                                                      "outputDataSE": "@{OutputSE}"}]
      assertEqualsImproved(self.lcs._listofoutput, expected_output_list, self)
      prodparam_dict = self.lcs.prodparameters
      assertEqualsImproved((prodparam_dict['detectorType'], prodparam_dict['lcsim_steeringfile'],
                              prodparam_dict['lcsim_trackingstrategy']), ('SID', '', 'lfn:/myTestStrat'),
                            self)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Unit tests for the Calibration application."""

from __future__ import print_function
#  import unittest
#  import pytest
#  import os
#  import shutil
#  import time
#  from datetime import datetime
#  from datetime import timedelta
#  from xml.etree import ElementTree as et
#  from shutil import copyfile
#  from DIRAC import S_OK, S_ERROR
#  from mock import patch
#  from mock import MagicMock as Mock
from __future__ import absolute_import
from ILCDIRAC.Interfaces.API.NewInterface.Applications.Calibration import Calibration

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.Calibration'


def test_init():
  """Test initialization."""
  calApp = Calibration()
  assert calApp._modulename == "Calibration"
  assert calApp.appname == "marlin"


def test_setCalibrationID():
  """Test setCalibrationID function."""
  calApp = Calibration()

  # correct input
  arg = 3
  assert not calApp._errorDict
  calApp.setCalibrationID(arg)
  assert not calApp._errorDict

  # wrong input
  arg = 'some string'
  calApp.setCalibrationID(arg)
  assert calApp._errorDict


def test_setWorkerID():
  """Test setCalibrationID function."""
  calApp = Calibration()

  # correct input
  arg = 3
  assert not calApp._errorDict
  calApp.setWorkerID(arg)
  assert not calApp._errorDict

  # wrong input
  arg = 'some string'
  calApp.setWorkerID(arg)
  assert calApp._errorDict



#!/usr/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the ProcessList module."""

from __future__ import absolute_import
import unittest
from mock import patch, MagicMock as Mock, mock_open

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Core.Utilities.ProcessList import ProcessList
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertListContentEquals, \
    assertDiracFailsWith, assertDiracSucceeds, assertDiracSucceedsWith_equals, assertMockCalls
from six.moves import map

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Core.Utilities.ProcessList'

STD_PROC_DICT = {'TarBallCSPath': '/test/cs/path/ball.tar', 'Detail': 'TestNoDetails',
                  'Generator': 'mytestGen21', 'Model': 'testmodel3001', 'Restrictions': '',
                  'InFile': 'my/file.in'}


# pylint: disable=protected-access
class ProcessListSimpleTestCase(unittest.TestCase):
  """Test the simple methods of the class that dont need a sane CFG."""

  def setUp(self):
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=False)):
      self.prol = ProcessList('myTestProcess.list')

  def test_constructor(self):
    import diraccfg
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)), \
         patch.object(diraccfg.CFG, 'loadFromFile', return_value=True):
      empty_process_list = ProcessList('existent_location')
      self.assertTrue(empty_process_list.cfg.existsKey('Processes'))
      self.assertTrue(empty_process_list.isOK())

    def replace_load(self, _):  # pylint: disable=missing-docstring
      self.createNewSection('myTestSection', 'testComment')
      self.createNewSection('Processes', 'testProcesses')

    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)), \
         patch.object(diraccfg.CFG, 'loadFromFile', side_effect=replace_load, autospec=True):
      other_process_list = ProcessList('existent_location')
      self.assertTrue(other_process_list.cfg.existsKey('Processes'))
    self.assertFalse(self.prol.isOK())

  def test_addentry(self):
    self.prol.cfg.createNewSection('Processes')
    self.prol.cfg.createNewSection('Processes/123')
    self.prol._addEntry('123', STD_PROC_DICT)


class ProcessListComplexTestCase(unittest.TestCase):
  """Test the different methods of the class, providing a usable CFG."""

  def setUp(self):
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=False)):
      self.prol = ProcessList('myTestProcess.list')
    self.prol.cfg.createNewSection('Processes')

  def test_updateproclist_and_getters(self):
    self.prol.cfg.createNewSection('Processes/myTestProcDeleteMe')
    self.prol.cfg.setOption('Processes/myTestProcDeleteMe/someRandomOption', True)
    dict_1 = {'CrossSection': 'someCross'}
    dict_1.update(STD_PROC_DICT)
    dict_2 = {'CrossSection': 'some_other_val'}
    dict_2.update(STD_PROC_DICT)
    process_dict = {}
    process_dict['MytestProcess'] = dict_1
    process_dict['myTestProcDeleteMe'] = dict_2
    result = self.prol.updateProcessList(process_dict)
    assertDiracSucceeds(result, self)
    conf = self.prol.cfg
    self.assertFalse(conf.existsKey('Processes/myTestProcDeleteMe/someRandomOption'))
    options = ['Processes/MytestProcess/CrossSection', 'Processes/myTestProcDeleteMe/CrossSection']
    assertEqualsImproved((list(map(conf.getOption, options))), (['someCross', 'some_other_val']), self)
    assertEqualsImproved((self.prol.getCSPath('myTestProcDeleteMe'),
                            self.prol.getInFile('myTestProcDeleteMe'),
                            self.prol.existsProcess('myTestProcDeleteMe'),
                            self.prol.existsProcess(''), self.prol.existsProcess('invalidProcess'),
                            self.prol.existsProcess('myTestProcDeleteMeToo')),
                          ('/test/cs/path/ball.tar', 'my/file.in', S_OK(True), S_OK(True), S_OK(False),
                            S_OK(False)), self)
    assertListContentEquals(self.prol.getProcesses(), ['myTestProcDeleteMe', 'MytestProcess'], self)
    all_processes_dict = self.prol.getProcessesDict()
    assertEqualsImproved(len(all_processes_dict), 2, self)
    assertEqualsImproved(('myTestProcDeleteMe' in all_processes_dict, 'MytestProcess' in all_processes_dict),
                          (True, True), self)
    self.prol.printProcesses()

  def test_writeproclist(self):
    expected_write = 'Processes\n{\n  mytestprocess123\n  {\n    TarBallCSPath = /test/cs/path/bal.tarr\n    Detail = TestNoDetails\n    Generator = mytestGen21\n    Model = testmodel3001\n    Restrictions = \n    InFile = my/file.in\n    CrossSection = 0\n  }\n}\n'
    self.prol._addEntry('mytestprocess123', {'TarBallCSPath': '/test/cs/path/bal.tarr', 'Detail':
                                               'TestNoDetails', 'Generator': 'mytestGen21', 'Model':
                                               'testmodel3001', 'Restrictions': '', 'InFile': 'my/file.in'
                                             })
    exists_dict = {'/temp/dir': False, '/temp/dir/mytempfile.txt': True, '/my/folder/testpath.xml': True}
    with patch('tempfile.mkstemp', new=Mock(return_value=('handle', '/temp/dir/mytempfile.txt'))), \
         patch('diraccfg.cfg.open', mock_open()) as file_mock, \
         patch('os.makedirs') as mkdir_mock, \
         patch('os.path.exists', new=Mock(side_effect=lambda path: exists_dict[path])), \
         patch('shutil.move') as move_mock, \
         patch('os.close') as close_mock:
      assertDiracSucceedsWith_equals(self.prol.writeProcessList('/my/folder/testpath.xml'),
                                     '/my/folder/testpath.xml', self)
      mkdir_mock.assert_called_once_with('/temp/dir')
      file_mock.assert_called_once_with('/temp/dir/mytempfile.txt', 'w')
      file_mock().write.assert_called_once_with(expected_write)
      close_mock.assert_called_once_with('handle')
      move_mock.assert_called_once_with('/temp/dir/mytempfile.txt', '/my/folder/testpath.xml')

  def test_writeproclist_notwritten(self):
    exists_dict = {'myTmpNameTestme': True}
    cfg_mock = Mock()
    cfg_mock.writeToFile.return_value = False
    self.prol.cfg = cfg_mock
    self.prol.location = '/my/folder/testpath2.txt'
    with patch('os.close') as close_mock, \
         patch('os.path.exists', new=Mock(side_effect=lambda path: exists_dict[path])), \
         patch('os.remove') as remove_mock, \
         patch('tempfile.mkstemp', new=Mock(return_value=('myhandle', 'myTmpNameTestme'))):
      assertDiracFailsWith(self.prol.writeProcessList(), 'failed to write repo', self)
      close_mock.assert_called_once_with('myhandle')
      remove_mock.assert_called_once_with('myTmpNameTestme')

  def test_writeproclist_notwritten_noremove(self):
    exists_dict = {'myTmpNameTestme': False}
    cfg_mock = Mock()
    cfg_mock.writeToFile.return_value = False
    self.prol.cfg = cfg_mock
    with patch('os.close') as close_mock, \
         patch('os.path.exists', new=Mock(side_effect=lambda path: exists_dict[path])), \
         patch('os.remove') as remove_mock, \
         patch('tempfile.mkstemp', new=Mock(return_value=('myhandle', 'myTmpNameTestme'))):
      assertDiracFailsWith(self.prol.writeProcessList('/my/folder/testpath2.txt'),
                            'failed to write repo', self)
      close_mock.assert_called_once_with('myhandle')
      self.assertFalse(remove_mock.called)

  def test_writeproclist_move_fails(self):
    exists_dict = {'/my/folder/testpath2.txt': False}
    cfg_mock = Mock()
    cfg_mock.writeToFile.return_value = True
    self.prol.cfg = cfg_mock
    with patch('os.close') as close_mock, \
         patch('os.path.exists', new=Mock(side_effect=lambda path: exists_dict[path])), \
         patch('os.remove') as remove_mock, \
         patch('tempfile.mkstemp', new=Mock(return_value=('myhandle', 'myTmpNameTestme'))), \
         patch('shutil.move', new=Mock(side_effect=OSError('mytestErr_os'))):
      assertDiracFailsWith(self.prol.writeProcessList('/my/folder/testpath2.txt'),
                            'failed to write repo', self)
      close_mock.assert_called_once_with('myhandle')
      self.assertFalse(remove_mock.called)

  def test_uploadproclist(self):
    import sys
    import DIRAC
    datman_mock = Mock()
    datman_mock.removeFile.return_value = S_OK('something')
    datmodule_mock = Mock()
    datmodule_mock.DataManager.return_value = datman_mock
    fileutil_mock = Mock()
    fileutil_mock.upload.return_value = S_OK('something')
    conf_mock = Mock()
    conf_mock.getOption.return_value = S_OK('/local/path/proc.list')
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': datmodule_mock,
                       'ILCDIRAC.Core.Utilities.FileUtils': fileutil_mock}
    module_patcher = patch.dict(sys.modules, mocked_modules)
    module_patcher.start()
    backup_conf = DIRAC.gConfig
    DIRAC.gConfig = conf_mock
    with patch('shutil.copy') as copy_mock, \
         patch('subprocess.call') as proc_mock:
      self.prol.uploadProcessListToFileCatalog('/my/secret/path/processlist.whiz', 'v120')
      assertMockCalls(copy_mock, [
          ('myTestProcess.list', '/afs/cern.ch/eng/clic/software/whizard/whizard_195/'),
          ('myTestProcess.list', '/local/path/proc.list')], self)
      proc_mock.assert_called_once_with(
          ['svn', 'ci', '/afs/cern.ch/eng/clic/software/whizard/whizard_195/proc.list',
           "-m'Process list for whizard version v120'"], shell=False)
    DIRAC.gConfig = backup_conf
    module_patcher.stop()

  def test_uploadproclist_remove_fails(self):
    import sys
    import DIRAC
    datman_mock = Mock()
    datman_mock.removeFile.return_value = S_ERROR('my_test_err')
    datmodule_mock = Mock()
    datmodule_mock.DataManager.return_value = datman_mock
    fileutil_mock = Mock()
    conf_mock = Mock()
    conf_mock.getOption.return_value = S_OK('somepath')
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': datmodule_mock,
                       'ILCDIRAC.Core.Utilities.FileUtils': fileutil_mock}
    module_patcher = patch.dict(sys.modules, mocked_modules)
    module_patcher.start()
    backup_conf = DIRAC.gConfig
    DIRAC.gConfig = conf_mock
    DIRAC.exit = abort_test
    with self.assertRaises(KeyboardInterrupt) as ki:
      self.prol.uploadProcessListToFileCatalog('asd', 'v1')
    key_interrupt = ki.exception
    assertEqualsImproved(key_interrupt.args, ('abort_my_test', ), self)
    DIRAC.gConfig = backup_conf
    module_patcher.stop()

  def test_uploadproclist_upload_fails(self):
    import sys
    import DIRAC
    datman_mock = Mock()
    datman_mock.removeFile.return_value = S_OK('something')
    datmodule_mock = Mock()
    datmodule_mock.DataManager.return_value = datman_mock
    fileutil_mock = Mock()
    fileutil_mock.upload.return_value = S_ERROR('something')
    conf_mock = Mock()
    conf_mock.getOption.return_value = S_OK('somepath')
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': datmodule_mock,
                       'ILCDIRAC.Core.Utilities.FileUtils': fileutil_mock}
    module_patcher = patch.dict(sys.modules, mocked_modules)
    module_patcher.start()
    backup_conf = DIRAC.gConfig
    DIRAC.gConfig = conf_mock
    DIRAC.exit = abort_test
    with self.assertRaises(KeyboardInterrupt) as ki:
      self.prol.uploadProcessListToFileCatalog('asd', 'v1')
    key_interrupt = ki.exception
    assertEqualsImproved(key_interrupt.args, ('abort_my_test', ), self)
    DIRAC.gConfig = backup_conf
    module_patcher.stop()

  def test_uploadproclist_copy_and_commit_fail(self):
    import sys
    import DIRAC
    datman_mock = Mock()
    datman_mock.removeFile.return_value = S_OK('something')
    datmodule_mock = Mock()
    datmodule_mock.DataManager.return_value = datman_mock
    fileutil_mock = Mock()
    fileutil_mock.upload.return_value = S_OK('something')
    conf_mock = Mock()
    conf_mock.getOption.return_value = S_OK('somepath')
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': datmodule_mock,
                       'ILCDIRAC.Core.Utilities.FileUtils': fileutil_mock}
    module_patcher = patch.dict(sys.modules, mocked_modules)
    module_patcher.start()
    backup_conf = DIRAC.gConfig
    DIRAC.gConfig = conf_mock
    DIRAC.exit = abort_test
    with patch('shutil.copy', new=Mock(side_effect=OSError('oserr_testme_keeprunning'))), \
         patch('subprocess.call', new=Mock(side_effect=OSError('subproc_test_err'))):
      self.prol.uploadProcessListToFileCatalog('/my/secret/path/processlist.whiz', 'v120')
    DIRAC.gConfig = backup_conf
    module_patcher.stop()

  def test_uploadproclist_skip_copy(self):
    import sys
    import DIRAC
    datman_mock = Mock()
    datman_mock.removeFile.return_value = S_OK('something')
    datmodule_mock = Mock()
    datmodule_mock.DataManager.return_value = datman_mock
    fileutil_mock = Mock()
    fileutil_mock.upload.return_value = S_OK('something')
    conf_mock = Mock()
    conf_mock.getOption.return_value = S_OK('')
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': datmodule_mock,
                       'ILCDIRAC.Core.Utilities.FileUtils': fileutil_mock}
    module_patcher = patch.dict(sys.modules, mocked_modules)
    module_patcher.start()
    backup_conf = DIRAC.gConfig
    DIRAC.gConfig = conf_mock
    DIRAC.exit = abort_test
    with patch('shutil.copy', new=Mock(side_effect=IOError('dont_call_me'))), \
         patch('subprocess.call', new=Mock(side_effect=IOError('dont_call_me_either'))):
      self.prol.uploadProcessListToFileCatalog('/my/secret/path/processlist.whiz', 'v120')
    DIRAC.gConfig = backup_conf
    module_patcher.stop()


def abort_test(_):
  """Replaces DIRACs own exit method to be testable."""
  raise KeyboardInterrupt('abort_my_test')



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Run Whizard2.

:author: Marko Petric
:since:  June 29, 2015
"""

from __future__ import absolute_import
import os

from DIRAC.Core.Utilities.Subprocess import shellCall
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
from DIRAC import S_OK, S_ERROR, gLogger
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getEnvironmentScript, extractTarball
from ILCDIRAC.Core.Utilities.resolvePathsAndNames import getProdFilename

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class Whizard2Analysis(ModuleBase):
  """Specific Module to run a Whizard2 job."""

  def __init__(self):
    super(Whizard2Analysis, self).__init__()
    self.enable = True
    self.STEP_NUMBER = ''
    self.result = S_ERROR()
    self.applicationName = 'whizard2'
    self.startFrom = 0
    self.randomSeed = -1
    self.whizard2SinFile = ''
    self.whizard2RawSin = False
    self.eventstring = ['+++ Generating event']
    self.decayProc = ['decay_proc']
    self.integratedProcess = ''
    self.datMan = DataManager()

  def applicationSpecificInputs(self):
    """Resolve all input variables for the module here.

    :return: S_OK()
    """
    self.randomSeed = self._determineRandomSeed()

    if "IS_PROD" in self.workflow_commons and self.workflow_commons["IS_PROD"]:
      self.OutputFile = getProdFilename(self.OutputFile,
                                        int(self.workflow_commons["PRODUCTION_ID"]),
                                        int(self.workflow_commons["JOB_ID"]),
                                        self.workflow_commons,
                                       )

    return S_OK('Parameters resolved')

  def resolveIntegratedProcess(self):
    """Check if integrated process is set and act accordingly.

    If the integrated process was given as a tarball it should already be available in the working directory and we do
    nothing.
    """
    if not self.integratedProcess:
      return S_OK()

    # integratedProcess is set, check CVMFS or filecatalog
    processes = self.ops.getOptionsDict('/AvailableTarBalls/%s/whizard2/%s/integrated_processes/processes' %
                                        ('x86_64-slc5-gcc43-opt', self.applicationVersion))

    if not processes['OK']:
      LOG.error('Could not resolve known integrated processes', processes['Message'])
      return processes

    options = self.ops.getOptionsDict('/AvailableTarBalls/%s/whizard2/%s/integrated_processes' %
                                      ('x86_64-slc5-gcc43-opt', self.applicationVersion))
    if not options['OK']:
      LOG.error('Failed to get integrated processes options', options['Message'])
      return options

    cvmfsPath = options['Value'].get('CVMFSPath', '')
    tarballURL = options['Value'].get('TarBallURL', '')
    processTarball = processes['Value'].get(self.integratedProcess, '')

    localTarball = os.path.join(cvmfsPath, processTarball)
    if os.path.exists(localTarball):
      LOG.info('Tarball found on cvmfs: %r' % localTarball)
      return extractTarball(localTarball, os.getcwd())

    tarballLFN = os.path.join(tarballURL, processTarball)
    LOG.info('Trying to download tarball', tarballLFN)
    getFile = self.datMan.getFile(tarballLFN)
    if not getFile['OK']:
      LOG.error('Failed to download tarball', getFile['Message'])
      return getFile
    return extractTarball(os.path.split(tarballLFN)[1], os.getcwd())

  def runIt(self):
    """Called by JobAgent.

    Execute the following:
      - get the environment variables that should have been set during installation
      - prepare the steering file and command line parameters
      - run Whizard2 on this steering file and catch the exit status

    :rtype: :func:`~DIRAC.Core.Utilities.ReturnValues.S_OK`, :func:`~DIRAC.Core.Utilities.ReturnValues.S_ERROR`
    """
    self.result = S_OK()
    if not self.platform:
      self.result = S_ERROR('No ILC platform selected')
    elif not self.applicationLog:
      self.result = S_ERROR('No Log file provided')
    if not self.result['OK']:
      LOG.error("Failed to resolve input parameters:", self.result['Message'])
      return self.result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('Whizard2 should not proceed as previous step did not end properly')

    resIntProc = self.resolveIntegratedProcess()
    if not resIntProc['OK']:
      return resIntProc

    # get the enviroment script
    res = getEnvironmentScript(
        self.platform,
        self.applicationName,
        self.applicationVersion,
        S_ERROR("No init script provided in CVMFS!"))
    if not res['OK']:
      LOG.error("Could not obtain the environment script: ", res["Message"])
      return res
    envScriptPath = res["Value"]

    whizard2SteerName = 'Whizard2_%s_Steer_%s.sin' % (self.applicationVersion, self.STEP_NUMBER)
    if os.path.exists(whizard2SteerName):
      os.remove(whizard2SteerName)

    whizard2Steer = []
    whizard2Steer.append('!Seed set via API')
    whizard2Steer.append('seed = %s' % self.randomSeed)
    whizard2Steer.append('')
    whizard2Steer.append('!Parameters set via whizard2SinFile')
    whizard2Steer.append('')
    whizard2Steer.append('')
    whizard2Steer.append(self.whizard2SinFile)
    whizard2Steer.append('')
    if not self.whizard2RawSin:
      whizard2Steer.append('!Number of events set via API')
      whizard2Steer.append('')
      whizard2Steer.append('n_events = %s' % self.NumberOfEvents)
      whizard2Steer.append('')
      whizard2Steer.append('simulate (%s) {' % ','.join(self.decayProc))
      whizard2Steer.append('        $sample = "%s"' % self.OutputFile.rsplit('.', 1)[0])
      if self.OutputFile.rsplit('.', 1)[-1] == 'slcio':
        whizard2Steer.append('        sample_format = lcio')
        whizard2Steer.append('        $extension_lcio = "slcio"')
      else:
        whizard2Steer.append('        sample_format = %s' % self.OutputFile.rsplit('.', 1)[-1])
        whizard2Steer.append('        $extension_{st} = "{st}"'.format(st=self.OutputFile.rsplit('.', 1)[-1]))
      whizard2Steer.append('}')

    with open(whizard2SteerName, 'w') as steerFile:
      steerFile.write("\n".join(whizard2Steer))

    scriptName = 'Whizard2_%s_Run_%s.sh' % (self.applicationVersion, self.STEP_NUMBER)
    if os.path.exists(scriptName):
      os.remove(scriptName)
    script = []
    script.append('#!/bin/bash')
    script.append('#####################################################################')
    script.append('# Dynamically generated script to run a production or analysis job. #')
    script.append('#####################################################################')
    script.append('source %s' % envScriptPath)
    script.append('echo =========')
    script.append('env | sort >> localEnv.log')
    script.append('echo whizard:`which whizard`')
    script.append('echo =========')
    script.append('whizard %s' % whizard2SteerName)
    script.append('declare -x appstatus=$?')
    script.append('exit $appstatus')

    with open(scriptName, 'w') as scriptFile:
      scriptFile.write("\n".join(script))

    if os.path.exists(self.applicationLog):
      os.remove(self.applicationLog)

    os.chmod(scriptName, 0o755)
    comm = 'bash "./%s"' % scriptName
    self.setApplicationStatus('Whizard2 %s step %s' % (self.applicationVersion, self.STEP_NUMBER))
    self.stdError = ''
    self.result = shellCall(0, comm, callbackFunction=self.redirectLogOutput, bufferLimit=20971520)
    resultTuple = self.result['Value']
    if not os.path.exists(self.applicationLog):
      LOG.error("Something went terribly wrong, the log file is not present")
      self.setApplicationStatus('%s failed to produce log file' % (self.applicationName))
      if not self.ignoreapperrors:
        return S_ERROR('%s did not produce the expected log %s' % (self.applicationName, self.applicationLog))
    status = resultTuple[0]

    LOG.info("Status after the application execution is %s" % status)

    return self.finalStatusReport(status)

  def _determineRandomSeed(self):
    """determine what the randomSeed should be, depends on production or not.

    .. Note::
      Whizard2 we use *randomSeed* and not *RandomSeed* as in the other workflow modules
    """
    if self.randomSeed == -1:
      self.randomSeed = self.jobID
    if "IS_PROD" in self.workflow_commons:
      self.randomSeed = int(
          str(
              int(
                  self.workflow_commons["PRODUCTION_ID"]))
          + str(
              int(
                  self.workflow_commons["JOB_ID"])))
    return self.randomSeed



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""
Mokka: Simulation after Whizard or StdHepCut
"""

from __future__ import absolute_import
import os
import types

from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.Workflow.Parameter import Parameter

from ILCDIRAC.Interfaces.API.NewInterface.LCApplication import LCApplication
import six

LOG = gLogger.getSubLogger(__name__)
__RCSID__ = "$Id$"


class Mokka(LCApplication):
  """Call Mokka simulator (after Whizard, Pythia or StdHepCut)

  To ensure reproductibility, the randomSeed is used as mcRunNumber. By default it's the jobID.

  Usage:

  >>> wh = Whizard()
  ...
  >>> mo = Mokka()
  >>> mo.getInputFromApp(wh)
  >>> mo.setSteeringFile("mysteer.steer")
  >>> mo.setMacFile('MyMacFile.mac')
  >>> mo.setStartFrom(10)

  Use :func:`setExtraCLIArguments` if you want to pass command line arguments to Mokka
  """

  def __init__(self, paramdict=None):

    self.startFrom = 0
    self.macFile = ''
    self.randomSeed = 0
    self.mcRunNumber = 0
    self.dbSlice = ''
    self.detectorModel = ''
    self.processID = ''
    super(Mokka, self).__init__(paramdict)
    # Those 5 need to come after default constructor
    self._modulename = 'MokkaAnalysis'
    self._moduledescription = 'Module to run MOKKA'
    self.appname = 'mokka'
    self.datatype = 'SIM'
    self.detectortype = 'ILD'
    self._paramsToExclude.extend(["outputDstPath", "outputRecPath", "OutputDstFile", "OutputRecFile"])

  def setRandomSeed(self, randomSeed):
    """ Optional: Define random seed to use. Default is JobID.

    Also used as *mcRunNumber*.

    :param int randomSeed: Seed to use during integration and generation. Default is Job ID.
    """
    self._checkArgs({'randomSeed': int})

    self.randomSeed = randomSeed

  def setmcRunNumber(self, runnumber):
    """ Optional: Define mcRunNumber to use. Default is 0. In Production jobs, is equal to RandomSeed

    :param int runnumber: mcRunNumber parameter of Mokka
    """
    self._checkArgs({'runnumber': int})

    self.mcRunNumber = runnumber

  def setDetectorModel(self, detectorModel):
    """Define detector to use for Mokka simulation.

    :param str detectorModel: Detector Model to use for Mokka simulation.
    """
    self._checkArgs({'detectorModel': (str,)})

    self.detectorModel = detectorModel

  def setMacFile(self, macfile):
    """ Optional: Define Mac File. Useful if using particle gun.

    :param str macfile: Macro file for Mokka
    """
    self._checkArgs({'macfile': (str,)})
    self.macFile = macfile
    if os.path.exists(macfile) or macfile.lower().count("lfn:"):
      self.inputSB.append(macfile)
    elif self.macFile:
      LOG.notice("Mac file not found locally and is not an lfn, I hope you know what you are doing...")
      LOG.notice("MacFile:", self.macFile)
    else:
      pass

  def setStartFrom(self, startfrom):
    """ Optional: Define from where mokka starts to read in the generator file

    :param int startfrom: from which event mokka starts to read the input file
    """
    self._checkArgs({'startfrom': int})
    self.startFrom = startfrom

  def setProcessID(self, processID):
    """ Optional: Define the processID. This is added to the event header.

    :param str processID: process ID string

    """
    self._checkArgs({'processID': (str,)})
    self.processID = processID

  def setDbSlice(self, dbSlice):
    """ Optional: Define the data base that will use mokka

    :param str dbSlice: database used by Mokka

    """
    self._checkArgs({'dbSlice': (str,)})
    self.dbSlice = dbSlice
    if os.path.exists(dbSlice) or dbSlice.lower().count("lfn:"):
      self.inputSB.append(dbSlice)
    elif dbSlice:
      LOG.notice("Slice not found locally and is not an lfn, I hope you know what you are doing...")
      LOG.notice("DB slice:", self.dbSlice)
    else:
      pass

  def _userjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setUserJobFinalization(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('userjobmodules failed')
    return S_OK()

  def _prodjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setOutputComputeDataList(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('prodjobmodules failed')
    return S_OK()

  def _checkConsistency(self, job=None):

    if not self.version:
      return S_ERROR('No version found')

    if not self.steeringFile:
      return S_ERROR('No Steering File')

    # FIXME: delete dead code
    # if not os.path.exists(self.steeringFile) and not self.steeringFile.lower().count("lfn:"):
      ##res = Exists(self.SteeringFile)
      #res = S_OK()
      # if not res['OK']:
      # return res

    #res = self._checkRequiredApp()
    # if not res['OK']:
    #  return res

    if self._jobtype != 'User':
      self._listofoutput.append({"outputFile": "@{OutputFile}",
                                 "outputPath": "@{OutputPath}",
                                 "outputDataSE": '@{OutputSE}'})
      self.prodparameters['mokka_steeringfile'] = self.steeringFile
      if self.detectorModel:
        self.prodparameters['mokka_detectormodel'] = self.detectorModel
      self.prodparameters['detectorType'] = self.detectortype

    return S_OK()

  def _applicationModule(self):

    md1 = self._createModuleDefinition()
    md1.addParameter(Parameter("RandomSeed", 0, "int", "", "", False, False,
                               "Random seed for the generator"))
    md1.addParameter(Parameter("mcRunNumber", 0, "int", "", "", False, False,
                               "mcRunNumber parameter for Mokka"))
    md1.addParameter(Parameter("detectorModel", "", "string", "", "", False, False,
                               "Detector model for simulation"))
    md1.addParameter(Parameter("macFile", "", "string", "", "", False, False, "Mac file"))
    md1.addParameter(Parameter("startFrom", 0, "int", "", "", False, False,
                               "From where Mokka start to read the input file"))
    md1.addParameter(Parameter("dbSlice", "", "string", "", "", False, False, "Data base used"))
    md1.addParameter(Parameter("ProcessID", "", "string", "", "", False, False, "Process ID"))
    md1.addParameter(Parameter("debug", False, "bool", "", "", False, False, "debug mode"))
    return md1

  def _applicationModuleValues(self, moduleinstance):

    moduleinstance.setValue("RandomSeed", self.randomSeed)
    moduleinstance.setValue("detectorModel", self.detectorModel)
    moduleinstance.setValue("mcRunNumber", self.mcRunNumber)
    moduleinstance.setValue("macFile", self.macFile)
    moduleinstance.setValue("startFrom", self.startFrom)
    moduleinstance.setValue("dbSlice", self.dbSlice)
    moduleinstance.setValue("ProcessID", self.processID)
    moduleinstance.setValue("debug", self.debug)

  def _checkWorkflowConsistency(self):
    return self._checkRequiredApp()

  def _resolveLinkedStepParameters(self, stepinstance):
    if isinstance(self._linkedidx, six.integer_types):
      self._inputappstep = self._jobsteps[self._linkedidx]
    if self._inputappstep:
      stepinstance.setLink("InputFile", self._inputappstep.getType(), "OutputFile")
    return S_OK()



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the GenericApplication module."""

from __future__ import absolute_import
import unittest

from mock import patch, MagicMock as Mock
from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Applications.GenericApplication import GenericApplication
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertMockCalls

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.GenericApplication'

# pylint: disable=protected-access


class GenericAppTestCase(unittest.TestCase):
  """Test the GenericApplication class."""

  def setUp(self):
    """set up the objects."""
    self.gapp = GenericApplication({})

  def test_setters(self):
    self.assertFalse(self.gapp._errorDict)
    assertDiracSucceeds(self.gapp.setScript('lfn:/my/script.sh'), self)
    assertDiracSucceeds(self.gapp.setArguments('arg1 arg2 -t arg3 /h?'), self)
    assertDiracSucceeds(self.gapp.setDependency({'mylibrary': 'v1.0test', 'mokka': 'v1002'}), self)
    self.assertFalse(self.gapp._errorDict)
    assertEqualsImproved((self.gapp.script, self.gapp.inputSB, self.gapp.arguments, self.gapp.dependencies),
                         ('lfn:/my/script.sh', ['lfn:/my/script.sh'], 'arg1 arg2 -t arg3 /h?',
                          {'mylibrary': 'v1.0test', 'mokka': 'v1002'}), self)

  def test_setters_typeerrors(self):
    self.assertFalse(self.gapp._errorDict)
    self.gapp.setScript('')
    self.gapp.setArguments({})
    self.gapp.setDependency([])
    assertEqualsImproved(len(self.gapp._errorDict['_checkArgs']), 2, self)

  def test_setscript(self):
    self.assertFalse(self.gapp._errorDict)
    self.gapp.setScript('/invalid/dir/myscript.sh')
    self.assertFalse(self.gapp._errorDict)
    assertEqualsImproved((self.gapp.script, self.gapp.inputSB), ('/invalid/dir/myscript.sh', []), self)

  def test_applicationModule(self):
    ret_mock = Mock()
    with patch.object(self.gapp, '_createModuleDefinition', new=Mock(return_value=ret_mock)) as mod_mock:
      assertEqualsImproved(self.gapp._applicationModule(), ret_mock, self)
      self.assertTrue(mod_mock.called)
      self.assertTrue(ret_mock.addParameter.called)

  def test_applicationmodulevalues(self):
    mod_mock = Mock()
    self.gapp._applicationModuleValues(mod_mock)
    self.assertTrue(mod_mock.setValue.called)

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.gapp._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.gapp._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
            patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.gapp._userjobmodules(None),
                           'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_OK('something'))), \
            patch('%s._setOutputComputeDataList' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_other_test_err'))):
      assertDiracFailsWith(self.gapp._prodjobmodules(None),
                           'prodjobmodules failed', self)

  def test_addparams(self):
    assertDiracSucceeds(self.gapp._addParametersToStep(Mock()), self)

  def test_addparams_fails(self):
    with patch.object(self.gapp, '_addBaseParameters', new=Mock(return_value=S_ERROR('bla'))):
      assertDiracFailsWith(self.gapp._addParametersToStep(Mock()), 'failed to set base param', self)

  def test_setstepparams(self):
    job_mock = Mock()
    self.gapp._job = job_mock
    self.gapp.dependencies = {'mydependency': 'v1', 'other_library': 'v100testme'}
    self.gapp._setStepParametersValues(Mock())
    assertMockCalls(job_mock._addSoftware, [('mydependency', 'v1'), ('other_library', 'v100testme')], self)

  def test_checkconsistency(self):
    self.gapp.script = 'lfn:/totally/valid/path.sh'
    assertDiracSucceeds(self.gapp._checkConsistency(), self)

  def test_checkconsistency_noscript(self):
    self.gapp.script = ''
    assertDiracFailsWith(self.gapp._checkConsistency(), 'script not def', self)

  def test_checkconsistency_scriptnotfound(self):
    self.gapp.script = '/invalid/path/dontfindme.sh'
    assertDiracFailsWith(self.gapp._checkConsistency(), 'script is not an lfn and was not found', self)

  def test_checkconsistency_setupscriptnotfound(self):
    self.gapp.script = 'lfn:/totally/valid/path.sh'
    self.gapp.setupScript = '/invalid/path/dontfindme.sh'
    assertDiracFailsWith(
        self.gapp._checkConsistency(),
        'specified source script is not an LFN and was not found on disk',
        self)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test ProductionJob."""

from __future__ import print_function
from __future__ import absolute_import
import textwrap
import unittest
from decimal import Decimal
from mock import patch, mock_open, MagicMock as Mock

from parameterized import parameterized

from DIRAC import gLogger, S_OK, S_ERROR
from DIRAC.Core.Security.ProxyInfo import getProxyInfo
from ILCDIRAC.Interfaces.API.NewInterface.ProductionJob import ProductionJob
from ILCDIRAC.Interfaces.API.NewInterface.Applications.DDSim import DDSim
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds
from Tests.Utilities.FileUtils import FileUtil
import six

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.ProductionJob'
DDSIM_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.DDSim'
BUILTIN_NAME = 'builtins' if six.PY3 else '__builtin__'

gLogger.setLevel("DEBUG")
gLogger.showHeaders(True)


class ProductionJobTestCase(unittest.TestCase):
  """Base class for the ProductionJob test cases."""

  def setUp(self):
    """set up the objects."""
    super(ProductionJobTestCase, self).setUp()
    with patch.object(ProductionJob, 'setPlatform', new=Mock(S_OK())):
      self.prodJob = ProductionJob()
    self.prodJob.trc = Mock(name="TRC")
    self.prodJob.trc.getTransformationStats.return_value = S_ERROR('Production does not exist yet')
    self.prodJob.energy = 250.0
    self.prodJob.fc = Mock('FileCatalogMock')
    self.prodJob.fc.isDirectory = Mock(name='isDir')
    self.prodJob.fc.isDirectory.side_effect = isdir_sideeffect
    self.prodJob.fc.getMetadataFields = Mock(name='getMeta')
    self.prodJob.fc.createDirectory = Mock(name='mkdir')
    self.prodJob.fc.changePathMode = Mock(name='chmod')
    self.prodJob.fc.setMetadata = Mock(name='SMD')
    self.prodJob.fc.getDirectoryUserMetadata = Mock(name='GDUMD')

  def test_Energy250(self):
    """ProductionJob getEnergyPath 250gev..........................................................."""
    self.prodJob.energy = Decimal('250.0')
    res = self.prodJob.getEnergyPath()
    self.assertEqual("250gev/", res)

  def test_Energy350(self):
    """ProductionJob getEnergyPath 350gev..........................................................."""
    self.prodJob.energy = 350.0
    res = self.prodJob.getEnergyPath()
    self.assertEqual("350gev/", res)

  def test_Energy3000(self):
    """ProductionJob getEnergyPatt 3tev............................................................."""
    self.prodJob.energy = 3000.0
    res = self.prodJob.getEnergyPath()
    self.assertEqual("3tev/", res)

  def test_Energy1400(self):
    """ProductionJob getEnergyPath 1.4tev .........................................................."""
    self.prodJob.energy = 1400.0
    res = self.prodJob.getEnergyPath()
    self.assertEqual("1.4tev/", res)

  def test_Energy43222(self):
    """Test ProductionJob getEnergyPath 432.22gev."""
    self.prodJob.energy = 432.22
    res = self.prodJob.getEnergyPath()
    self.assertEqual("432.22gev/", res)


# FIXME Add checks that the important lines were really executed
class ProductionJobCompleteTestCase(unittest.TestCase):
  """Tests the rest of the ProductionJob TestCases."""

  def setUp(self):
    super(ProductionJobCompleteTestCase, self).setUp()
    with patch.object(ProductionJob, 'setPlatform', new=Mock(S_OK())):
      self.prodJob = ProductionJob()

  def test_setconfig(self):
    # TODO add more checks on the result, espc. if the addParameter call was successful
    ver = '1481.30'
    res = self.prodJob.setConfig(ver)
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.prodparameters['ILDConfigVersion'], ver, self)

  def test_setclicconfig(self):
    ver = '2017-99-99'
    res = self.prodJob.setCLICConfig(ver)
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.prodparameters['ClicConfigVersion'], ver, self)


class ProductionJobSetJobFileGroupSizeTest(ProductionJobTestCase):
  """Tests the setJobFileGroupSize method."""

  def test_setJobFileGroupSize_normal(self):
    # Basic setter method
    num = 4871
    self.prodJob.setJobFileGroupSize(num)
    assertEqualsImproved(self.prodJob.jobFileGroupSize, num, self)
    assertEqualsImproved(self.prodJob.prodparameters['NbInputFiles'], num, self)

  def test_setJobFileGroupSize_fails(self):
    # Append before changing jobFileGroupSize, causing it to fail
    # Application can be arbitrary
    ddsim = DDSim()
    ddsim.setVersion('ILCSoft-01-17-09')
    ddsim.setDetectorModel('CLIC_o2_v03')
    ddsim.setNumberOfEvents(1)
    ddsim.setInputFile('Muon_50GeV_Fixed_cosTheta0.7.stdhep')
    # Set necessary parameters to call append() - job is never run
    self.prodJob.energy = 250.0
    self.prodJob.evttype = 'electron party'
    self.prodJob.outputStorage = 'CERN-EOS-DST'
    with patch('%s._analyseJob' % DDSIM_NAME, new=Mock(return_value=S_OK())), \
         patch('%s._checkConsistency' % DDSIM_NAME, new=Mock(return_value=S_OK())), \
         patch('%s._checkFinalConsistency' % DDSIM_NAME, new=Mock(return_value=S_OK())):
      res = self.prodJob.append(ddsim)
      assertDiracSucceeds(res, self)
    res = self.prodJob.setJobFileGroupSize(1389)
    assertDiracFailsWith(res, 'input is needed at the beginning', self)


class ProductionJobSetInputDataQuery(ProductionJobTestCase):
  """Tests the setInputDataQuery method."""

  @classmethod
  def setUpClass(cls):
    ProductionJobSetInputDataQuery.HAS_PROXY = getProxyInfo()['OK']

  def test_setInputDataQuery(self):
    """Test the inputDataQuery method."""
    self.assertFalse(self.prodJob.dryrun)
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK({'DirectoryMetaFields': {'ProdID': 19872456}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'EvtType': 'electron party'}))
    self.prodJob.energycat = '7'
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456})
    assertDiracSucceeds(res, self)
    # TODO check output of method
    assertEqualsImproved(self.prodJob.energy, Decimal('7'), self)

  def test_setInputDataQuery_dryrun(self):
    """Test the inputDataQuery method."""
    self.prodJob.dryrun = True
    self.assertTrue(self.prodJob.dryrun)
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK({'DirectoryMetaFields': {'ProdID': 'int',
                                                                                        'EvtType': 'string'}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({0: 'None'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({}))
    self.prodJob.energycat = '7'
    res = self.prodJob.setInputDataQuery({'ProdID': 1007, 'EvtType': 'Martini'})
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.energy, Decimal('7'), self)

  # TODO fix here
  def test_setInputDataQuery_getmetadatafails(self):
    """Test the inputDataQuery when getmetadata fails."""
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_ERROR('some_error'))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456})
    assertDiracFailsWith(res, 'could not contact file catalog', self)

  def test_setInputDataQuery_filecatalogWrongCase(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK({'DirectoryMetaFields': {'prodid': 19872456}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456})
    assertDiracFailsWith(res, 'key syntax error', self)

  def test_setInputDataQuery_filecatalogMissingKey(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK({'DirectoryMetaFields': {'ProdID': 19872456}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'EvtType': 'electron party'}))
    # Key not present in getMetadataFields return value
    randomkey = '09428ituj42itufgm'
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456, randomkey: 'testvalue'})
    assertDiracFailsWith(res, 'key %r not found in metadata keys' % randomkey, self)

  def test_setInputDataQuery_noprodid(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK({'DirectoryMetaFields': {'ProdID': 19872456}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(side_effect=[S_OK(['dir1', 'dir2']), S_OK({'abc': 'testdir123'})])
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'EvtType': 'electron party'}))
    res = self.prodJob.setInputDataQuery({})
    assertDiracFailsWith(res, "input metadata dictionary must contain at least a key 'prodid' as reference",
                         self)

  @parameterized.expand([(S_ERROR('some_error'), 'error looking up the catalog'),
                         (S_OK({}), 'could not find any directories'),
                         (S_OK({0: 'None'}), 'could not find any directories')])
  def test_setInputDataQuery_finddir_fails(self, retVal, expectedError):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK({'DirectoryMetaFields': {'ProdID': 19872456}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=retVal)
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456})
    assertDiracFailsWith(res, expectedError, self)

  def test_setInputDataQuery_getdirusermetadata_fails(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK({'DirectoryMetaFields': {'ProdID': 19872456}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_ERROR('some_error'))
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456})
    assertDiracFailsWith(res, 'error looking up the catalog for directory metadata', self)

  def test_setInputDataQuery_getenergyfromcompatmeta_1(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK({'DirectoryMetaFields': {'ProdID': 19872456}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(
        return_value=S_OK({'EvtType': ['electron party'], 'Energy': '13gev'}))
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456})
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.energy, Decimal('13'), self)

  def test_setInputDataQuery_getenergyfromcompatmeta_2(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK({'DirectoryMetaFields': {'ProdID': 19872456}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(
        return_value=S_OK({'EvtType': 'electron party', 'Energy': ['13tev']}))
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456})
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.energy, Decimal('13000'), self)

  def test_setInputDataQuery_getenergyfromcompatmeta_3(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK({'DirectoryMetaFields': {'ProdID': 19872456}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'EvtType': 'electron party', 'Energy': 13}))
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456})
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.energy, Decimal('13'), self)

  def test_setInputDataQuery_noevttype(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK({'DirectoryMetaFields': {'ProdID': 19872456}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({}))
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456})
    assertDiracFailsWith(res, 'evttype is not in the metadata', self)

  def test_setInputDataQuery_numofevts_1(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK(
        {'DirectoryMetaFields': {'ProdID': 19872456, 'NumberOfEvents': 'testsuihe123'}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'EvtType': 'electron party', 'Energy': 13}))
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456, 'NumberOfEvents': ['42985']})
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.nbevts, 42985, self)

  def test_setInputDataQuery_numofevts_2(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK(
        {'DirectoryMetaFields': {'ProdID': 19872456, 'NumberOfEvents': 'testabc'}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'EvtType': 'electron party', 'Energy': 13}))
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456, 'NumberOfEvents': '968541'})
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.nbevts, 968541, self)

  def test_setInputDataQuery_datatype_1(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK(
        {'DirectoryMetaFields': {'ProdID': 19872456, 'Datatype': 'test123', 'DetectorType': 'testdetector'}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'EvtType': 'electron party'}))
    self.prodJob.energycat = '7'
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456, 'Datatype': 'mytype',
                                          'DetectorType': 'GoodDetector874'})
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.datatype, 'mytype', self)
    assertEqualsImproved(self.prodJob.detector, 'GoodDetector874', self)

  def test_setInputDataQuery_datatype_2(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK(
        {'DirectoryMetaFields': {'ProdID': 19872456, 'Datatype': 'test123', 'DetectorType': 'testdetector'}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'EvtType': 'electron party'}))
    self.prodJob.energycat = '7'
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456, 'Datatype': 'gen', 'DetectorType': 'abc'})
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.datatype, 'gen', self)
    assertEqualsImproved(self.prodJob.detector, '', self)

  def test_setInputDataQuery_datatype_list1(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK(
        {'DirectoryMetaFields': {'ProdID': 19872456, 'Datatype': 'test123', 'DetectorType': 'testdetector'}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'EvtType': 'electron party'}))
    self.prodJob.energycat = '7'
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456, 'Datatype': ['mytype'],
                                          'DetectorType': ['MyDetector3000']})
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.datatype, 'mytype', self)
    assertEqualsImproved(self.prodJob.detector, 'MyDetector3000', self)

  def test_setInputDataQuery_datatype_list2(self):
    self.prodJob.fc.getMetadataFields = Mock(return_value=S_OK(
        {'DirectoryMetaFields': {'ProdID': 19872456, 'Datatype': 'test123', 'DetectorType': 'testdetector'}}))
    self.prodJob.fc.findDirectoriesByMetadata = Mock(return_value=S_OK({'abc': 'testdir123'}))
    self.prodJob.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'EvtType': 'electron party'}))
    self.prodJob.energycat = '7'
    res = self.prodJob.setInputDataQuery({'ProdID': 19872456, 'Datatype': ['gen'],
                                          'DetectorType': '904215fadf'})
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.datatype, 'gen', self)
    assertEqualsImproved(self.prodJob.detector, '', self)

  def test_createproduction(self):
    job = self.prodJob
    job.proxyinfo = {'OK': 'yes, trust me', 'Value': {'groupProperties': 'ProductionManagement'}}
    job.created = False
    job.inputdataquery = True
    job.slicesize = 10
    job.inputBKSelection = {'ProdID': 19872456}
    job.call_finalization = True
    job.prodparameters['ILDConfigVersion'] = 'goodversion1.215'
    job.nbevts = 89134
    job.finalpaths = ['test/path/my']
    job.workflow.setName('mytestworkflow')
    job.finalsdict = {'uploadData': 'myuploaddata', 'registerData': 'myregisterdata',
                       'uploadLog': 'myuploadlog', 'sendFailover': 'mysendfailover'}
    file_contents = [["I'm an XML file"]]
    handles = FileUtil.getMultipleReadHandles(file_contents)

    with patch('%s.open' % BUILTIN_NAME, mock_open()) as all_mo, \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo, \
         patch('%s.Transformation.addTransformation' % MODULE_NAME, new=Mock(return_value=S_OK())):
      mo.side_effect = (h for h in handles)
      job.description = 'MyTestDescription'
      res = job.createProduction('goodtestname')
      assertDiracSucceeds(res, self)
      trans = res['Value']
      assert trans.inputMetaQuery == {'ProdID': 19872456}
      expected = [[]]
      FileUtil.checkFileInteractions(self, mo, [('mytestworkflow.xml', 'r')], expected, handles)
      all_mo.assert_any_call('Name.xml', 'w')
      all_mo_handle = all_mo()
      assert all_mo_handle.write.call_args[0][0].splitlines() == (EXPECTED_XML).splitlines()


  def test_createproduction_2(self):
    job = self.prodJob
    job.proxyinfo = {'OK': 'yes, trust me', 'Value': {'groupProperties': 'ProductionManagement'}}
    job.created = False
    job.inputdataquery = True
    job.slicesize = 10
    job.inputBKSelection = True
    job.call_finalization = True
    job.finalpaths = ['test/path/my']
    job.workflow.setName('mytestworkflow')
    job.finalsdict = {'uploadData': 'myuploaddata', 'registerData': 'myregisterdata',
                       'uploadLog': 'myuploadlog', 'sendFailover': 'mysendfailover'}
    file_contents = [["I'm an XML file"]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % BUILTIN_NAME, mock_open()) as all_mo, \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo, \
         patch('%s.Transformation.addTransformation' % MODULE_NAME, new=Mock(return_value=S_OK())):
      mo.side_effect = (h for h in handles)
      job.description = 'MyTestDescription'
      res = job.createProduction('goodtestname')
      assertDiracSucceeds(res, self)
      expected = [[]]
      expected_tuples = [('mytestworkflow.xml', 'r')]
      FileUtil.checkFileInteractions(self, mo, expected_tuples, expected, handles)
      all_mo.assert_any_call('Name.xml', 'w')
      all_mo_handle = all_mo()
      assert all_mo_handle.write.call_args[0][0].splitlines() == (EXPECTED_XML).splitlines()

  def test_createproduction_nofinalization(self):
    job = self.prodJob
    job.proxyinfo = {'OK': 'yes, trust me', 'Value': {'groupProperties': 'ProductionManagement'}}
    job.created = False
    job.call_finalization = False
    job.workflow.setName('mytestworkflow')
    job.finalsdict = {'uploadData': 'myuploaddata', 'registerData': 'myregisterdata',
                       'uploadLog': 'myuploadlog', 'sendFailover': 'mysendfailover'}
    file_contents = [["I'm an XML file"]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % BUILTIN_NAME, mock_open()) as all_mo, \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo, \
         patch('%s.Transformation.addTransformation' % MODULE_NAME, new=Mock(return_value=S_OK())):
      mo.side_effect = (h for h in handles)
      job.description = 'MyTestDescription'
      res = job.createProduction()
      assertDiracSucceeds(res, self)
      expected = [[]]
      expected_tuples = [('mytestworkflow.xml', 'r')]
      FileUtil.checkFileInteractions(self, mo, expected_tuples, expected, handles)
      all_mo.assert_any_call('Name.xml', 'w')
      all_mo_handle = all_mo()
      assert all_mo_handle.write.call_args[0][0].splitlines() == (EXPECTED_XML_NOFINAL).splitlines()

  @patch('%s.open' % BUILTIN_NAME, mock_open())
  def test_createproduction_basic_checks(self):
    job = self.prodJob
    job.proxyinfo = {'OK': False, 'Message': 'not ok'}
    res = job.createProduction()
    assertDiracFailsWith(res, 'you need a production proxy', self)
    job.proxyinfo = {'OK': True, 'Value': {}}
    res = job.createProduction()
    assertDiracFailsWith(res, 'could not determine group', self)
    job.proxyinfo = {'OK': True, 'Value': {'groupProperties': 'NormalUser'}}
    res = job.createProduction()
    assertDiracFailsWith(res, 'not allowed to create production', self)
    job.created = True
    job.proxyinfo = {'OK': True, 'Value': {'groupProperties': 'ProductionManagement'}}
    res = job.createProduction()
    assertDiracFailsWith(res, 'already created', self)
    job.created = False
    with patch('%s.ProductionJob._addToWorkflow' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_error'))):
      assertDiracFailsWith(job.createProduction(), 'some_error', self)
    with patch('%s.ProductionJob.createWorkflow' % MODULE_NAME, new=Mock(side_effect=OSError('some_os_error'))):
      assertDiracFailsWith(job.createProduction(), 'could not create workflow', self)
    with patch('%s.Transformation.addTransformation' % MODULE_NAME, new=Mock(return_value=S_ERROR('myerror123'))), \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True):
      assertDiracFailsWith(job.createProduction(), 'myerror123', self)
    job.trc = Mock()
    # S_OK because it means it found a transformation by that name, so the new one cannot be created
    job.trc.getTransformationStats.return_value = S_OK('fail this')
    with patch('%s.open' % MODULE_NAME, mock_open(), create=True):
      assertDiracFailsWith(job.createProduction(), 'already exists', self)

  @patch('%s.open' % BUILTIN_NAME, mock_open())
  def test_createproduction_dryrun(self):
    job = self.prodJob
    job.proxyinfo = {'OK': 'yes, trust me', 'Value': {'groupProperties': 'ProductionManagement'}}
    job.created = False
    job.dryrun = True
    job.call_finalization = True
    job.workflow.setName('mytestworkflow')
    job.finalsdict = {'uploadData': 'myuploaddata', 'registerData': 'myregisterdata',
                       'uploadLog': 'myuploadlog', 'sendFailover': 'mysendfailover'}
    file_contents = [["I'm an XML file"]]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo, \
         patch('%s.Transformation.addTransformation' % MODULE_NAME, new=Mock(return_value=S_OK())):
      mo.side_effect = (h for h in handles)
      job.description = 'MyTestDescription'
      res = job.createProduction('goodtestname')
      assertDiracSucceeds(res, self)
      expected = [[]]
      FileUtil.checkFileInteractions(self, mo, [('mytestworkflow.xml', 'r')], expected, handles)

  def test_setNbOfTasks(self):
    assertDiracFailsWith(self.prodJob.setNbOfTasks(5), 'no transformation defined', self)
    self.prodJob.currtrans = Mock()
    self.prodJob.inputBKSelection = True
    # Returns empty S_ERROR, probably should add at least error message
    assertDiracFailsWith(self.prodJob.setNbOfTasks(2), '', self)
    self.prodJob.inputBKSelection = False
    testNbTasks = 1375
    res = self.prodJob.setNbOfTasks(testNbTasks)
    assertDiracSucceeds(res, self)
    assertEqualsImproved(self.prodJob.nbtasks, testNbTasks, self)
    self.prodJob.currtrans.setMaxNumberOfTasks.assert_called_with(testNbTasks)  # pylint: disable=no-member

  # TODO Fix runtime of these tests???
  def test_finalizeProd(self):
    job = self.prodJob
    job.slicesize = 0
    job.dryrun = False
    testdict = {'JobType': 'mytest', 'Process': 'mytestprocess', 'Energy': 86451}
    job.currtrans = Mock()
    job.currtrans.getTransformationID.return_value = {'Value': 651}  # pylint: disable=no-member
    res = job.finalizeProd(1387, testdict)
    assertDiracSucceeds(res, self)

  def test_finalizeProd_dryrun(self):
    job = self.prodJob
    job.slicesize = 0
    testdict = {'JobType': 'mytest', 'Process': 'mytestprocess', 'Energy': 86451}
    job.currtrans = 651
    job.dryrun = True
    res = job.finalizeProd(1387, testdict)
    assertDiracSucceeds(res, self)

  def test_finalizeProd_withparams(self):
    job = self.prodJob
    job.slicesize = 0
    job.prodparameters = {'JobType': ['mytest'], 'lumi': 12, 'NbInputFiles': 1,
                           'FCInputQuery': {'sampleKey': 'sampleValue'}, 'SWPackages': 'mytestpackages',
                           'SoftwareTag': 'Monday', 'ILDConfigVersion': 'goodILDConfversion123.2'}
    job.prodparameters['ClicConfigVersion'] = 'better3.14'
    job.prodparameters['extraCLIArguments'] = '--Config.Tracking=Awesome'
    job.finalpaths = ['testpath123/a/b/c', 'othertestpath/many_dirs/file.txt']
    job.slicesize = 561
    job.metadict_external = {'additional_entry': 'swpackage_value'}
    job.finalMetaDict = {'testpath123/a/b/c': {},
                         'another_one/asd': {'key': 'val'},
                         'need/more/paths': {},
                         'nonsearchable/_exists_': {},
                         'othertestpath/many_dirs/file.txt': {}}
    job.finalMetaDictNonSearch = {'nonsearchable/path2': {},
                                  'test/path/more/needed': {}}

    job.fc.createDirectory.side_effect = createdir_sideeffect
    job.fc.changePathMode.side_effect = changepath_sideeffect
    job.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'key': 'val', 'k2': 'v2'}))

    job.trc.setTransformationParameter.return_value = S_OK(True)
    res = job.finalizeProd(1387)
    assertDiracSucceeds(res, self)
    # TODO Check result variables to be as expected

  def test_finalizeProd_metaFail(self):
    job = self.prodJob
    job.prodparameters = {'JobType': ['mytest'], 'lumi': 12, 'NbInputFiles': 1,
                          'FCInputQuery': {'sampleKey': 'sampleValue'},
                          'SWPackages': 'mytestpackages',
                          'SoftwareTag': 'Monday',
                          'ILDConfigVersion': 'goodILDConfversion123.2'}
    job.slicesize = 0
    job.finalMetaDict = {'another_one/asd': {'key': 'val'}}

    job.fc.createDirectory.side_effect = createdir_sideeffect
    job.fc.changePathMode.side_effect = changepath_sideeffect
    job.fc.getDirectoryUserMetadata = Mock(return_value=S_OK({'key': 'val2'}))

    job.trc.setTransformationParameter.return_value = S_OK(True)
    res = job.finalizeProd(1387)
    assertDiracFailsWith(res, 'meta', self)

  def test_finalizeProd_noswpackages_nometadictexternal(self):
    job = self.prodJob
    job.slicesize = 0
    job.prodparameters = {'JobType': 'mytest', 'lumi': 12, 'NbInputFiles': 1,
                           'FCInputQuery': {'sampleKey': 'sampleValue'}, 'SoftwareTag': 'Monday',
                           'ILDConfigVersion': 'goodILDConfversion123.2'}
    job.finalpaths = ['testpath123/a/b/c', 'othertestpath/many_dirs/file.txt']
    job.slicesize = 561
    job.finalMetaDict = {'asd': {'tag': 'asd'}}
    job.finalMetaDictNonSearch = {'testpath123/a/b/c': {}}

    job.fc.createDirectory.side_effect = createdir_sideeffect
    job.fc.changePathMode.side_effect = changepath_sideeffect

    job.trc.setTransformationParameter.return_value = S_OK(True)
    res = job.finalizeProd(1387)
    assertDiracSucceeds(res, self)

  def test_finalizeProd_lumiZero(self):
    job = self.prodJob
    job.prodparameters = {'JobType': 'mytest', 'lumi': 0, 'NbInputFiles': 1,
                           'FCInputQuery': {'sampleKey': 'sampleValue'}, 'SWPackages': 'mytestpackages',
                           'SoftwareTag': 'Monday', 'ILDConfigVersion': 'goodILDConfversion123.2'}
    res = job.finalizeProd(1387)
    assertDiracSucceeds(res, self)

  def test_finalizeProd_notrans(self):
    job = self.prodJob
    assertDiracFailsWith(job.finalizeProd(), 'no transformation defined', self)

  def test_finalizeProd_setMetaFails(self):
    job = self.prodJob
    job.fc.setMetadata = Mock(return_value=S_OK(True))
    job.slicesize = 0
    job.prodparameters = {'JobType': 'mytest', 'lumi': 12, 'NbInputFiles': 1,
                          'FCInputQuery': {'sampleKey': 'sampleValue'}, 'SWPackages': 'mytestpackages',
                          'SoftwareTag': 'Monday', 'ILDConfigVersion': 'goodILDConfversion123.2'}
    job.finalpaths = ['testpath123/a/b/c', 'othertestpath/many_dirs/file.txt']
    job.slicesize = 561
    job.metadict_external = {'additional_entry': 'swpackage_value'}
    job.finalMetaDict = {'testpath123/a/b/c': {}, 'another_path/file.txt': {}, 'another_one/asd': {},
                         'wrongpath': {}, 'something_invalid': {}, 'nonsearchable/path': {},
                         'other_unsearchable/path/f.txt': {}, 'need/more/paths': {},
                         'othertestpath/many_dirs/file.txt': {}}
    job.finalMetaDictNonSearch = {'nonsearchable/path2': {}, 'other_unsearchables/path/gh.txt': {},
                                  'test/path/more/needed': {}, 'my_file/hidden.txt': {}, 'tmp.txt': {},
                                  '/usr/bin/test': {}, '/myfile_f.txt': {}}

    job.fc.createDirectory = Mock(side_effect=createdir_sideeffect)
    job.fc.changePathMode = Mock(side_effect=changepath_sideeffect)
    res = job.finalizeProd(1387)
    assertDiracFailsWith(res, 'meta', self)

  def test_finalizeProd_setMetaFails_2(self):
    job = self.prodJob
    job.fc.setMetadata = Mock(return_value=S_ERROR('Failed to set meta'))
    job.slicesize = 0
    job.prodparameters = {'JobType': 'mytest', 'lumi': 12, 'NbInputFiles': 1,
                           'FCInputQuery': {'sampleKey': 'sampleValue'}, 'SWPackages': 'mytestpackages',
                           'SoftwareTag': 'Monday', 'ILDConfigVersion': 'goodILDConfversion123.2'}
    job.finalpaths = ['testpath123/a/b/c', 'othertestpath/many_dirs/file.txt']
    job.slicesize = 561
    job.finalMetaDict = {'testpath123/a/b/c': {'k': '1', 'k2': '2'}}

    job.fc.createDirectory = Mock(side_effect=createdir_sideeffect)
    job.fc.changePathMode = Mock(side_effect=changepath_sideeffect)
    res = job.finalizeProd(1387)
    assertDiracFailsWith(res, 'failed to set meta', self)

  def test_getMetadata(self):
    job = self.prodJob
    reference_dict = {'1': {'test1': 1, '09ksrt': '123tgvda'}, '2': {'vdunivi': -135, 21: 'sdfg',
                                                                          job: 0, 'NumberOfEvents': 1002}}
    reference_dict_nonbofevts = {'test1': 1, '09ksrt': '123tgvda', 'vdunivi': -135, 21: 'sdfg', job: 0}
    job.finalMetaDict = reference_dict
    assertEqualsImproved(job.getMetadata(), reference_dict_nonbofevts, self)
    job.finalMetaDict = {}
    assertEqualsImproved(job.getMetadata(), {}, self)
    easy_dict = {'key1': 'value1', '1983jrtmfgik': 1984137895, '198034': job, '09842jtm': '9k0femoqifu'}
    job.finalMetaDict = {'1': easy_dict}
    assertEqualsImproved(job.getMetadata(), easy_dict, self)


class ProductionJobJobSpecificParamsTest(ProductionJobTestCase):
  """Tests the jobSpecificParams method by calling append() and mocking out the other parts."""

  def setUp(self):
    """set up the objects."""
    super(ProductionJobJobSpecificParamsTest, self).setUp()
    self.prodJob.outputStorage = 'CERN-DIP-4'
    self.myapp = create_application_mock()

  def tearDown(self):
    self.prodJob = None
    self.myapp = None

  def test_jobSpecificParams(self):
    """Test jobSpecificParams inside the append call.

    Does nothing basically.
    """
    res = self.prodJob.append(self.myapp)
    assertDiracSucceeds(res, self)

  def test_jobSpecificParams_alreadyCreated(self):
    self.prodJob.created = True
    assertDiracFailsWith(self.prodJob.append(self.myapp), 'production was created', self)

  def test_jobSpecificParams_loggingFails(self):
    self.myapp.logFile = ''
    self.myapp.setLogFile.return_value = S_ERROR('some_log_error')
    res = self.prodJob.append(self.myapp)
    assertDiracFailsWith(res, 'some_log_error', self)

  def test_jobSpecificParams_checkNbEvts_1(self):
    self.prodJob.nbevts = 0
    self.prodJob.slicesize = 0
    assertDiracSucceeds(self.prodJob.append(self.myapp), self)
    assertEqualsImproved(self.prodJob.nbevts, 3, self)

  def test_jobSpecificParams_checkNbEvts_2(self):
    self.myapp.numberOfEvents = 0
    self.prodJob.nbevts = 0
    assertDiracFailsWith(self.prodJob.append(self.myapp), 'number of events to process is not defined', self)

  def test_jobSpecificParams_checkNbEvts_3(self):
    self.prodJob.slicesize = 6
    self.prodJob.nbevts = 0
    self.myapp.numberOfEvents = 0
    assertDiracSucceeds(self.prodJob.append(self.myapp), self)
    self.myapp.setNumberOfEvents.assert_called_with(6)

  def test_jobSpecificParams_checkNbEvts_4(self):
    self.prodJob.slicesize = 0
    self.prodJob.nbevts = 3
    self.prodJob.jobFileGroupSize = 7
    self.myapp.numberOfEvents = 0
    assertDiracSucceeds(self.prodJob.append(self.myapp), self)
    self.myapp.setNumberOfEvents.assert_called_with(21)

  def test_jobSpecificParams_checkNbEvts_5(self):
    self.prodJob.nbevts = 0
    self.prodJob.slicesize = 1
    self.myapp.numberOfEvents = 11
    assertDiracSucceeds(self.prodJob.append(self.myapp), self)

  def test_jobSpecificParams_checkNbEvts_6(self):
    self.prodJob.nbevts = 0
    self.prodJob.slicesize = 0
    self.myapp.numberOfEvents = 11
    assertDiracSucceeds(self.prodJob.append(self.myapp), self)

  def test_jobSpecificParams_checkNbEvts_7(self):
    self.prodJob.nbevts = 0
    self.prodJob.slicesize = 2
    self.myapp.numberOfEvents = 0
    self.myapp.setNumberOfEvents.return_value = S_ERROR('some_append_error')
    assertDiracFailsWith(self.prodJob.append(self.myapp), 'some_append_error', self)

  def test_jobSpecificParams_energytests(self):
    self.prodJob.energy = 0
    self.myapp.energy = 0
    assertDiracFailsWith(self.prodJob.append(self.myapp), 'could not find the energy defined', self)
    self.prodJob.energy = 250
    self.myapp.setEnergy.return_value = S_ERROR('some_energy_error')
    assertDiracFailsWith(self.prodJob.append(self.myapp), 'some_energy_error', self)
    with patch('%s.hasattr' % MODULE_NAME, new=Mock(return_value=False), create=True):
      self.prodJob.evttype = ''
      self.myapp.setEnergy.return_value = S_OK(True)
      assertDiracFailsWith(self.prodJob.append(self.myapp), 'event type not found', self)
    self.prodJob.prodparameters['SWPackages'] = 'someProgramv1.0;anotherProgram'
    reference_string = 'someProgramv1.0;anotherProgram;coolphysicssimulation.v3p10'
    self.myapp.accountInProduction = False
    self.assertTrue(self.prodJob.append(self.myapp))
    assertEqualsImproved(self.prodJob.prodparameters['SWPackages'], reference_string, self)
    with patch('%s.ProductionJob._updateProdParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_update_error'))):
      assertDiracFailsWith(self.prodJob.append(self.myapp), 'some_update_error', self)
    self.prodJob.outputStorage = ''
    assertDiracFailsWith(self.prodJob.append(self.myapp), 'specify the output storage element', self)

  def test_jobSpecificParams_failures(self):
    self.myapp.setOutputSE.return_value = S_ERROR('setOSE_error')
    assertDiracFailsWith(self.prodJob.append(self.myapp), 'setOSE_error', self)
    self.myapp.setOutputSE.return_value = S_OK(True)
    self.prodJob.evttype = 'mytype/'
    self.myapp.willBeCut = False
    with patch('%s.hasattr' % MODULE_NAME, new=Mock(return_value=True), create=True):
      self.assertTrue(self.prodJob.append(self.myapp))
    with patch('%s.hasattr' % MODULE_NAME, new=Mock(side_effect=[True, False, True, True, True, True]), create=True):
      self.myapp.outputFile = False
      self.myapp.willBeCut = False
      self.myapp.detectortype = 'mydetector'
      assertDiracSucceeds(self.prodJob.append(self.myapp), self)
    with patch('%s.hasattr' % MODULE_NAME, new=Mock(side_effect=[True, False, True, True, True, True]), create=True):
      my_dict = self.prodJob.prodparameters

      def getitem(name):
        return my_dict[name]

      def setitem(name, val):
        my_dict[name] = val
      mock_dict = Mock()
      mock_dict.__getitem__.side_effect = getitem
      mock_dict.__setitem__.side_effect = setitem
      mock_dict.update.side_effect = ValueError('some_prodparam_err')
      self.prodJob.prodparameters = mock_dict
      self.prodJob.evttype = ''
      self.myapp.outputFile = False
      self.myapp.willBeCut = False
      self.myapp.detectortype = ''
      self.myapp.datatype = ''
      self.prodJob.datatype = 'datatype'
      self.prodJob.detector = 'mycooldetector'
      assertDiracFailsWith(self.prodJob.append(self.myapp), 'some_prodparam_err', self)

  def hasAttrMocks(self, *args, **kwargs):  # pylint: disable=unused-argument
    """mock the operations getConfig calls."""
    print("arguments", args, kwargs)
    opsDict = {
        'extraCLIArguments': False,
        'detectorModel': False,
        'detectortype': False,
        'detector': False,
        'eventType': True,
        'outputFile': True,
        'datatype': True,
        'setOutputRecFile': False,
        'setOutputDstFile': False,
        }
    self.assertIn(args[1], opsDict)
    return opsDict[args[1]]

  def hasAttrMocks_2(self, *args, **kwargs):  # pylint: disable=unused-argument
    """mock the operations getConfig calls."""
    print("arguments", args, kwargs)
    opsDict = {
        'extraCLIArguments': False,
        'detectorModel': False,
        'detectortype': True,
        'detector': False,
        'eventType': True,
        'outputFile': True,
        'datatype': True,
        'setOutputRecFile': False,
        'setOutputDstFile': False,
        }
    self.assertIn(args[1], opsDict)
    return opsDict[args[1]]

  def test_jobSpecificParams_detector_and_datatype(self):
    self.myapp.setOutputSE.return_value = S_OK(True)
    self.prodJob.evttype = ''
    self.myapp.willBeCut = False
    self.myapp.outputFile = False
    self.myapp.extraCLIArguments = ''
    hasAttrMock = Mock()
    hasAttrMock.side_effect = self.hasAttrMocks
    with patch('%s.hasattr' % MODULE_NAME, new=hasAttrMock, create=True), \
         patch('%s.ProductionJob._updateProdParameters' % MODULE_NAME, new=Mock(return_value=S_OK())):
      assertDiracSucceeds(self.prodJob.append(self.myapp), self)
    self.prodJob.evttype = ''

    hasAttrMock = Mock()
    hasAttrMock.side_effect = self.hasAttrMocks_2
    with patch('%s.hasattr' % MODULE_NAME, new=hasAttrMock, create=True):
      self.myapp.detectortype = 'application_detectortype'
      assertDiracSucceeds(self.prodJob.append(self.myapp), self)

  @parameterized.expand([(True, ['/vo/prod/250gev/fake/awesome/REC', '/vo/prod/250gev/fake/awesome/DST']),
                         (False, ['/vo/prod/250gev/fake/awesome/DST'])])
  def test_jobSpecificParams_rec_and_dst(self, keep, finalPaths):
    """Test if disabling rec file works."""
    self.myapp.setOutputSE.return_value = S_OK(True)
    self.prodJob.evttype = ''
    self.prodJob.basepath = '/vo/prod/'
    self.prodJob.basename = 'basename'
    self.myapp.eventType = 'fake'
    self.myapp.detectortype = 'awesome'
    self.myapp.willBeCut = False
    self.myapp.outputFile = False
    self.myapp.extraCLIArguments = ''
    self.myapp.keepRecFile = keep
    self.myapp.setOutputRecFile = Mock(name='RecFile')
    self.myapp.setOutputDstFile = Mock(name='DstFile')
    with patch('%s.ProductionJob._updateProdParameters' % MODULE_NAME, new=Mock(return_value=S_OK())):
      assertDiracSucceeds(self.prodJob.append(self.myapp), self)
    if keep:
      self.myapp.setOutputRecFile.assert_called_once()
    else:
      self.myapp.setOutputRecFile.assert_not_called()
    self.myapp.setOutputDstFile.assert_called_once()
    assert self.prodJob.finalpaths == finalPaths

  def test_setters_1(self):
    import random
    self.prodJob.setDryRun(True)
    assertEqualsImproved(self.prodJob.dryrun, True, self)
    self.prodJob.setProdGroup('myprodgroup')
    assertEqualsImproved(self.prodJob.prodGroup, 'myprodgroup', self)
    self.prodJob.setProdPlugin('Limited')
    assertEqualsImproved(self.prodJob.plugin, 'Limited', self)
    self.prodJob.setNbEvtsPerSlice(13)
    assertEqualsImproved(self.prodJob.slicesize, 13, self)
    myprodtype = random.choice(ALLOWED_PRODTYPES)
    self.prodJob.setDescription("It's green")
    assert self.prodJob.description[-5:] == 'green'
    self.prodJob.setProdType(myprodtype)
    assertEqualsImproved(self.prodJob.type, myprodtype, self)
    self.prodJob.setWorkflowName('mysuperworkflow')
    assertEqualsImproved(self.prodJob.workflow['name'], 'mysuperworkflow', self)
    assertEqualsImproved(self.prodJob.workflow['name'], self.prodJob.name, self)
    self.prodJob.setWorkflowDescription('this is a test workflow, pls dont execute')
    assertEqualsImproved(self.prodJob.workflow['description'],
                          'this is a test workflow, pls dont execute', self)
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=[True, False])), \
         patch('%s.shutil.move' % MODULE_NAME, new=Mock(return_value=True)) as move_mock, \
         patch('%s.os.remove' % MODULE_NAME, new=Mock(True)) as remove_mock, \
         patch('%s.open' % BUILTIN_NAME, mock_open()) as open_mock:
      self.prodJob.createWorkflow()
      assertEqualsImproved(len(move_mock.mock_calls), 1, self)
      move_mock.assert_called_once_with('mysuperworkflow.xml', 'mysuperworkflow.xml.backup')
      self.assertFalse(remove_mock.called)
      assertEqualsImproved(len(open_mock.mock_calls), 4, self)  # Open, __enter__, write, __exit__
    assertDiracSucceeds(self.prodJob.setOutputSE('myoutputstorage_for_testing'), self)
    assertEqualsImproved(self.prodJob.outputStorage, 'myoutputstorage_for_testing', self)

  def test_setters_2(self):
    self.prodJob.setDryRun(False)
    assertEqualsImproved(self.prodJob.dryrun, False, self)
    self.prodJob.setProdGroup('')
    assertEqualsImproved(self.prodJob.prodGroup, '', self)
    self.prodJob.setProdPlugin('')
    assertEqualsImproved(self.prodJob.plugin, '', self)
    self.prodJob.setNbEvtsPerSlice(0)
    assertEqualsImproved(self.prodJob.slicesize, 0, self)
    # Throws exception
    try:
      self.prodJob.setProdType('9i8u1j4tn')
      self.fail()
    except TypeError as myerror:
      self.assertNotEqual(self.prodJob.type, '9i8u1j4tn')
      message = '%s' % myerror
      self.assertIn('prod must be one of', message.lower())
    self.prodJob.setWorkflowName('')
    assertEqualsImproved(self.prodJob.workflow['name'], '', self)
    assertEqualsImproved(self.prodJob.workflow['name'], self.prodJob.name, self)
    self.prodJob.setWorkflowDescription('')
    assertEqualsImproved(self.prodJob.workflow['description'], '', self)
    # Skipping createWorkflow
    self.prodJob.setOutputSE('')
    assertEqualsImproved(self.prodJob.outputStorage, '', self)


ALLOWED_PRODTYPES = ['MCGeneration', 'MCSimulation', 'Test', 'MCReconstruction', 'MCReconstruction_Overlay',
                     'Merge', 'Split']


def create_application_mock():
  """Returns a mock object containing all necessary values for being used as an application in ProductionJob."""
  myapp = Mock()
  myapp._analyseJob.return_value = S_OK()
  myapp._checkConsistency.return_value = S_OK()
  myapp._checkFinalConsistency.return_value = S_OK()
  myapp.inputSB = [[]]
  myapp.energy = 350  # gev
  myapp.numberOfEvents = 3
  myapp.appname = 'coolphysicssimulation'
  myapp.version = 'v3p10'
  return myapp


# Constants needed for the tests
EXPECTED_XML = '\n'.join(['<Workflow>',
                          '<descr_short></descr_short>',
                          '<description><![CDATA[]]></description>',
                          '<name>mytestworkflow</name>',
                          '<origin></origin>',
                          '<type></type>',
                          '<version>0.0</version>',
                          '<Parameter name="JobType" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Job Type"><value><![CDATA[User]]></value></Parameter>',
                          '<Parameter name="Priority" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Priority"><value><![CDATA[1]]></value></Parameter>',
                          '<Parameter name="JobGroup" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="User specified job group"><value><![CDATA[@{PRODUCTION_ID}]]></value></Parameter>',
                          '<Parameter name="JobName" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Name of Job"><value><![CDATA[Name]]></value></Parameter>',
                          '<Parameter name="StdOutput" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Standard output file"><value><![CDATA[std.out]]></value></Parameter>',
                          '<Parameter name="StdError" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Standard error file"><value><![CDATA[std.err]]></value></Parameter>',
                          '<Parameter name="InputData" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Default null input data value"><value><![CDATA[]]></value></Parameter>',
                          '<Parameter name="LogLevel" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="User specified logging level"><value><![CDATA[verbose]]></value></Parameter>',
                          '<Parameter name="arguments" type="string" linked_module="" linked_parameter="" in="True" out="False" description="Arguments to executable Step"><value><![CDATA[]]></value></Parameter>',
                          '<Parameter name="ParametricInputData" type="string" linked_module="" linked_parameter="" in="True" out="False" description="Default null parametric input data value"><value><![CDATA[]]></value></Parameter>',
                          '<Parameter name="ParametricInputSandbox" type="string" linked_module="" linked_parameter="" in="True" out="False" description="Default null parametric input sandbox value"><value><![CDATA[]]></value></Parameter>',
                          '<Parameter name="IS_PROD" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="This job is a production job"><value><![CDATA[True]]></value></Parameter>',
                          '<Parameter name="CPUTime" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="CPU time in secs"><value><![CDATA[300000]]></value></Parameter>',
                          '<Parameter name="productionVersion" type="string" linked_module="" linked_parameter="" in="True" out="False" description="ProdAPIVersion"><value><![CDATA[$Id$]]></value></Parameter>',
                          '<Parameter name="PRODUCTION_ID" type="string" linked_module="" linked_parameter="" in="True" out="False" description="ProductionID"><value><![CDATA[00012345]]></value></Parameter>',
                          '<Parameter name="JOB_ID" type="string" linked_module="" linked_parameter="" in="True" out="False" description="ProductionJobID"><value><![CDATA[00012345]]></value></Parameter>',
                          '<Parameter name="emailAddress" type="string" linked_module="" linked_parameter="" in="True" out="False" description="CrashEmailAddress"><value><![CDATA[ilcdirac-support@cern.ch]]></value></Parameter>',
                          '\n'.join(textwrap.dedent(modDef).strip() for modDef in [
                              """
                            <ModuleDefinition>
                            <body><![CDATA[from ILCDIRAC.Workflow.Modules.UploadOutputData import UploadOutputData]]></body>
                            <descr_short></descr_short>
                            <description><![CDATA[Uploads the output data]]></description>
                            <origin></origin>
                            <required></required>
                            <type>UploadOutputData</type>
                            <version>0.0</version>
                            <Parameter name="enable" type="bool" linked_module="" linked_parameter="" in="True" out="False" description="EnableFlag"><value><![CDATA[False]]></value></Parameter>
                            </ModuleDefinition>
                            """,
                              """
                            <ModuleDefinition>
                            <body><![CDATA[from ILCDIRAC.Workflow.Modules.RegisterOutputData import RegisterOutputData]]></body>
                            <descr_short></descr_short>
                            <description><![CDATA[Module to add in the metadata catalog the relevant info about the files]]></description>
                            <origin></origin>
                            <required></required>
                            <type>RegisterOutputData</type>
                            <version>0.0</version>
                            <Parameter name="enable" type="bool" linked_module="" linked_parameter="" in="True" out="False" description="EnableFlag"><value><![CDATA[False]]></value></Parameter>
                            </ModuleDefinition>
                            """,
                              """
                            <ModuleDefinition>
                            <body><![CDATA[from ILCDIRAC.Workflow.Modules.ReportErrors import ReportErrors]]></body>
                            <descr_short></descr_short>
                            <description><![CDATA[Reports errors at the end]]></description>
                            <origin></origin>
                            <required></required>
                            <type>ReportErrors</type>
                            <version>0.0</version>
                            </ModuleDefinition>
                            """ if six.PY2 else '',
                              """
                            <ModuleDefinition>
                            <body><![CDATA[from ILCDIRAC.Workflow.Modules.UploadLogFile import UploadLogFile]]></body>
                            <descr_short></descr_short>
                            <description><![CDATA[Uploads the output log files]]></description>
                            <origin></origin>
                            <required></required>
                            <type>UploadLogFile</type>
                            <version>0.0</version>
                            <Parameter name="enable" type="bool" linked_module="" linked_parameter="" in="True" out="False" description="EnableFlag"><value><![CDATA[False]]></value></Parameter>
                            </ModuleDefinition>
                            """,
                              """
                            <ModuleDefinition>
                            <body><![CDATA[from ILCDIRAC.Workflow.Modules.FailoverRequest import FailoverRequest]]></body>
                            <descr_short></descr_short>
                            <description><![CDATA[Sends any failover requests]]></description>
                            <origin></origin>
                            <required></required>
                            <type>FailoverRequest</type>
                            <version>0.0</version>
                            <Parameter name="enable" type="bool" linked_module="" linked_parameter="" in="True" out="False" description="EnableFlag"><value><![CDATA[False]]></value></Parameter>
                            </ModuleDefinition>
                            """,
                              """
                            <ModuleDefinition>
                            <body><![CDATA[from ILCDIRAC.Workflow.Modules.ReportErrors import ReportErrors]]></body>
                            <descr_short></descr_short>
                            <description><![CDATA[Reports errors at the end]]></description>
                            <origin></origin>
                            <required></required>
                            <type>ReportErrors</type>
                            <version>0.0</version>
                            </ModuleDefinition>
                            """ if six.PY3 else '',
                              ] if modDef),
                          '<StepDefinition>',
                          '<descr_short></descr_short>',
                          '<description><![CDATA[]]></description>',
                          '<origin></origin>',
                          '<type>Job_Finalization</type>',
                          '<version>0.0</version>',
                          '<ModuleInstance>',
                          '<descr_short></descr_short>',
                          '<name>dataUpload</name>',
                          '<type>UploadOutputData</type>',
                          '<Parameter name="enable" type="bool" linked_module="" linked_parameter="" in="True" out="False" description="EnableFlag"><value><![CDATA[True]]></value></Parameter>',
                          '</ModuleInstance>',
                          '<ModuleInstance>',
                          '<descr_short></descr_short>',
                          '<name>RegisterOutputData</name>',
                          '<type>RegisterOutputData</type>',
                          '<Parameter name="enable" type="bool" linked_module="" linked_parameter="" in="True" out="False" description="EnableFlag"><value><![CDATA[True]]></value></Parameter>',
                          '</ModuleInstance>',
                          '<ModuleInstance>',
                          '<descr_short></descr_short>',
                          '<name>logUpload</name>',
                          '<type>UploadLogFile</type>',
                          '<Parameter name="enable" type="bool" linked_module="" linked_parameter="" in="True" out="False" description="EnableFlag"><value><![CDATA[True]]></value></Parameter>',
                          '</ModuleInstance>',
                          '<ModuleInstance>',
                          '<descr_short></descr_short>',
                          '<name>failoverRequest</name>',
                          '<type>FailoverRequest</type>',
                          '<Parameter name="enable" type="bool" linked_module="" linked_parameter="" in="True" out="False" description="EnableFlag"><value><![CDATA[True]]></value></Parameter>',
                          '</ModuleInstance>',
                          '<ModuleInstance>',
                          '<descr_short></descr_short>',
                          '<name>reportErrors</name>',
                          '<type>ReportErrors</type>',
                          '</ModuleInstance>',
                          '</StepDefinition>',
                          '<StepInstance>',
                          '<descr_short></descr_short>',
                          '<name>finalization</name>',
                          '<type>Job_Finalization</type>',
                          '</StepInstance>',
                          '</Workflow>',
])


EXPECTED_XML_NOFINAL = '\n'.join(['<Workflow>',
                                  '<descr_short></descr_short>',
                                  '<description><![CDATA[]]></description>',
                                  '<name>mytestworkflow</name>',
                                  '<origin></origin>',
                                  '<type></type>',
                                  '<version>0.0</version>',
                                  '<Parameter name="JobType" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Job Type"><value><![CDATA[User]]></value></Parameter>',
                                  '<Parameter name="Priority" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Priority"><value><![CDATA[1]]></value></Parameter>',
                                  '<Parameter name="JobGroup" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="User specified job group"><value><![CDATA[@{PRODUCTION_ID}]]></value></Parameter>',
                                  '<Parameter name="JobName" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Name of Job"><value><![CDATA[Name]]></value></Parameter>',
                                  '<Parameter name="StdOutput" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Standard output file"><value><![CDATA[std.out]]></value></Parameter>',
                                  '<Parameter name="StdError" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Standard error file"><value><![CDATA[std.err]]></value></Parameter>',
                                  '<Parameter name="InputData" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="Default null input data value"><value><![CDATA[]]></value></Parameter>',
                                  '<Parameter name="LogLevel" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="User specified logging level"><value><![CDATA[verbose]]></value></Parameter>',
                                  '<Parameter name="arguments" type="string" linked_module="" linked_parameter="" in="True" out="False" description="Arguments to executable Step"><value><![CDATA[]]></value></Parameter>',
                                  '<Parameter name="ParametricInputData" type="string" linked_module="" linked_parameter="" in="True" out="False" description="Default null parametric input data value"><value><![CDATA[]]></value></Parameter>',
                                  '<Parameter name="ParametricInputSandbox" type="string" linked_module="" linked_parameter="" in="True" out="False" description="Default null parametric input sandbox value"><value><![CDATA[]]></value></Parameter>',
                                  '<Parameter name="IS_PROD" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="This job is a production job"><value><![CDATA[True]]></value></Parameter>',
                                  '<Parameter name="CPUTime" type="JDL" linked_module="" linked_parameter="" in="True" out="False" description="CPU time in secs"><value><![CDATA[300000]]></value></Parameter>',
                                  '<Parameter name="productionVersion" type="string" linked_module="" linked_parameter="" in="True" out="False" description="ProdAPIVersion"><value><![CDATA[$Id$]]></value></Parameter>',
                                  '<Parameter name="PRODUCTION_ID" type="string" linked_module="" linked_parameter="" in="True" out="False" description="ProductionID"><value><![CDATA[00012345]]></value></Parameter>',
                                  '<Parameter name="JOB_ID" type="string" linked_module="" linked_parameter="" in="True" out="False" description="ProductionJobID"><value><![CDATA[00012345]]></value></Parameter>',
                                  '<Parameter name="emailAddress" type="string" linked_module="" linked_parameter="" in="True" out="False" description="CrashEmailAddress"><value><![CDATA[ilcdirac-support@cern.ch]]></value></Parameter>',
                                  '</Workflow>',
])


CREATEDIR_DICT = {'testpath123/a/b/c': S_OK({'Successful': {'testpath123/a/b/c': 'created'}}),
                  'another_path/file.txt': S_OK({'Successful': {}, 'Failed':
                                                 {'another_path/file.txt':
                                                  'could not create, OSError'}}),
                  'another_one/asd': S_OK({'Successful': {'another_one/asf': 'created'}}),
                  'wrongpath': S_ERROR('some_error'),
                  'something_invalid': S_OK({'Successful': {}, 'Failed': {}}),
                  'nonsearchable/path': S_OK({'Successful': {'nonsearchable/path': 'created'}}),
                  'other_unsearchable/path/f.txt': S_OK({'Successful': {},
                                                         'Failed': {'other_unsearchable/path/f.ppt':
                                                                    'could not create, OSError'}}),
                  'need/more/paths': S_OK({'Successful': {'need/more/paths': 'created'}}),
                  'othertestpath/many_dirs/file.txt': S_OK({'Successful':
                                                            {'othertestpath/many_dirs/file.txt':
                                                             'created'}}),
                  'nonsearchable/path2': S_OK({'Successful': {'nonsearchable/path2': 'created'}}),
                  'other_unsearchables/path/gh.txt': S_OK({'Successful': {}, 'Failed':
                                                           {'other_unsearchables/path/gh.txt':
                                                            'could not create, OSError'}}),
                  'test/path/more/needed': S_OK({'Successful': {'diff_string': 'created'}}),
                  'my_file/hidden.txt': S_ERROR('some_error'),
                  'tmp.txt': S_OK({'Successful': {}, 'Failed': {}}),
                  '/usr/bin/test': S_OK({'Successful': {'/usr/bin/test': 'created'}}),
                  '/myfile_f.txt': S_OK({'Successful': {}, 'Failed':
                                         {'/myfile_f.ppt': 'could not create, OSError'}}),
                  'asd': S_OK({'Successful': {'asd': 'created'}, 'Failed': {}})}

CHANGEPATH_DICT = {'testpath123/a/b/c': S_OK(),
                    'nonsearchable/path': S_ERROR('this is a test. fail please.'),
                    'need/more/paths': S_OK(),
                    'othertestpath/many_dirs/file.txt': S_OK(),
                    'nonsearchable/path2': S_OK(),
                    '/usr/bin/test': S_ERROR(),
                    'another_one/asd': S_OK(),
                    'test/path/more/needed': S_OK(),
                    'asd': S_OK()}  # , S_OK(), S_ERROR('this is a test. fail please.')}


def isdir_sideeffect(path):
  """Return true or false."""
  if '_exists_' in path:
    return S_OK({'Successful': {path: True}, 'Failed': {}})
  return S_OK({'Successful': {}, 'Failed': {path: 'Missing'}})


def createdir_sideeffect(value):
  """Returns the appropriate return value of the createDir method for the given directory string.

  :param str value: directory to be created
  :returns: S_OK/S_ERROR structure with the value in CREATEDIR_DICT
  :rtype: dict
  """
  return CREATEDIR_DICT[value]


def changepath_sideeffect(val, bool_flag):  # pylint: disable=unused-argument
  """Returns the appropriate return value of the changePathMode method for the given directory string. bool_flag is not used in this test version, but in the original method.

  :param str val: path to be changed to
  :param bool bool_flag: ignored, used in the actual implementation
  :returns: S_OK/S_ERROR structure with the value in CHANGEPATH_DICT
  :rtype: dict
  """
  return CHANGEPATH_DICT[next(six.iterkeys(val))]


def runTests():
  """Runs our tests."""
  alltests = unittest.TestSuite()
  alltests.addTest(unittest.makeSuite(ProductionJobTestCase))
  alltests.addTest(unittest.makeSuite(ProductionJobCompleteTestCase))
  alltests.addTest(unittest.makeSuite(ProductionJobSetJobFileGroupSizeTest))
  alltests.addTest(unittest.makeSuite(ProductionJobSetInputDataQuery))
  alltests.addTest(unittest.makeSuite(ProductionJobJobSpecificParamsTest))
  testResult = unittest.TextTestRunner(verbosity=2).run(alltests)
  print(testResult)


if __name__ == '__main__':
  runTests()



#!/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Add specified software in CS. Allows not using the web interface, therefore reduces the error rate.

Created on May 5, 2010
"""

from __future__ import absolute_import
__RCSID__ = "$Id$"

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import gLogger, gConfig, S_OK, exit as dexit
import os
try:
  import hashlib as md5
except ImportError:
  import md5


class _Params(object):
  """Collection of Parameters set via CLI switches."""

  def __init__(self):
    self.version = ''
    self.platform = 'x86_64-slc5-gcc43-opt'
    self.comment = ''
    self.name = ''
    self.tarball = ''

  def setVersion(self, optionValue):
    self.version = optionValue
    return S_OK()

  def setPlatform(self, optionValue):
    self.platform = optionValue
    return S_OK()

  def setName(self, optionValue):
    self.name = optionValue
    return S_OK()

  def setComment(self, optionValue):
    self.comment = optionValue
    return S_OK()

  def setTarBall(self, option):
    self.tarball = option
    return S_OK()

  def registerSwitches(self):
    Script.registerSwitch("P:", "Platform=", "Platform ex. %s" % self.platform, self.setPlatform)
    Script.registerSwitch("N:", "Name=", "Application name", self.setName)
    Script.registerSwitch("V:", "Version=", "Version", self.setVersion)
    Script.registerSwitch("T:", "TarBall=", "Tar ball location", self.setTarBall)
    Script.registerSwitch("C:", "Comment=", "Comment", self.setComment)
    Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                         '\nUsage:',
                                         '  %s [option|cfgfile] ...\n' % Script.scriptName]))


class SoftwareAdder(object):
  """Container for all the objects and functions to add software to ILCDirac."""

  def __init__(self, platform, appName, tarball_loc, appVersion, comment):
    from DIRAC.Interfaces.API.DiracAdmin import DiracAdmin
    self.diracAdmin = DiracAdmin()
    self.modifiedCS = False
    self.appVersion = appVersion
    self.appName = appName
    self.platform = platform
    self.softSec = "/Operations/Defaults/AvailableTarBalls"

    self.appTar = self.checkForTarBall(tarball_loc)

    self.parameter = dict(softSec=self.softSec,
                           platform=self.platform,
                           appName=self.appName,
                           appname=self.appName.lower(),
                           appTar=self.appTar,
                           appTar_name=os.path.basename(self.appTar),
                           appVersion=self.appVersion,
                         )
    self.comment = comment
    self.mailadress = 'ilc-dirac@cern.ch'

  def checkConsistency(self):
    """checks if platform is defined, application exists, etc."""
    gLogger.notice("Checking consistency")
    av_platforms = gConfig.getSections(self.softSec, [])
    if av_platforms['OK']:
      if self.platform not in av_platforms['Value']:
        gLogger.error("Platform %s unknown, available are %s." % (self.platform, ", ".join(av_platforms['Value'])))
        gLogger.error("If yours is missing add it in CS")
        dexit(255)
    else:
      gLogger.error("Could not find all platforms available in CS")
      dexit(255)

    av_apps = gConfig.getSections("%(softSec)s/%(platform)s" % self.parameter, [])
    if not av_apps['OK']:
      gLogger.error("Could not find all applications available in CS")
      dexit(255)

  def commitToCS(self):
    """write changes to the CS to the server."""
    gLogger.notice("Commiting changes to the CS")
    if self.modifiedCS:
      result = self.diracAdmin.csCommitChanges(False)
      if not result['OK']:
        gLogger.error('Commit failed with message = %s' % (result['Message']))
        dexit(255)
      gLogger.info('Successfully committed changes to CS')
    else:
      gLogger.info('No modifications to CS required')

  def checkForTarBall(self, tarball_loc):
    """checks if the tarball exists."""
    gLogger.info("Check if tarball exists at %s" % tarball_loc)
    appTar = ''
    if tarball_loc:
      appTar = tarball_loc
      if self.appName == 'slic':
        self.appVersion = os.path.basename(tarball_loc).slit("_")[0].split("-")[1]
    else:
      if self.appVersion:
        appTar = "%s%s.tgz" % (self.appName, self.appVersion)
      else:
        gLogger.notice("Version not defined")

    if not os.path.exists(appTar):
      gLogger.error("Cannot find the file %s, exiting" % appTar)
      dexit(1)

    return appTar

  def notifyAboutNewSoftware(self):
    """Send an email to the mailing list if a new software version was defined."""

    # Only send email when something was actually added
    if not self.modifiedCS:
      return

    subject = '%s %s added to DIRAC CS' % (self.appName, self.appVersion)
    msg = 'New application %s %s declared into Configuration service\n %s' % (self.appName,
                                                                              self.appVersion,
                                                                              self.comment)
    from DIRAC.Core.Security.ProxyInfo import getProxyInfo
    from DIRAC.ConfigurationSystem.Client.Helpers.Registry import getUserOption
    from DIRAC.FrameworkSystem.Client.NotificationClient import NotificationClient

    notifyClient = NotificationClient()
    gLogger.notice('Sending mail for software installation to %s' % (self.mailadress))
    res = getProxyInfo()
    if not res['OK']:
      sender = 'ilcdirac-admin@cern.ch'
    else:
      if 'username' in res['Value']:
        sender = getUserOption(res['Value']['username'], 'Email')
      else:
        sender = 'nobody@cern.ch'
    gLogger.info('*' * 80)  # surround email with stars
    res = notifyClient.sendMail(self.mailadress, subject, msg, sender, localAttempt=False)
    gLogger.info('*' * 80)
    if not res['OK']:
      gLogger.error('The mail could not be sent: %s' % res['Message'])

  def uploadTarBall(self):
    """get the tarballURL from the CS and upload the tarball there.

    Exits when error is encountered
    """
    gLogger.notice("Uploading TarBall to the Grid")
    from ILCDIRAC.Core.Utilities.FileUtils import upload
    tarballurl = gConfig.getOption("%(softSec)s/%(platform)s/%(appname)s/TarBallURL" % self.parameter, "")
    if not tarballurl['OK'] or not tarballurl['Value']:
      gLogger.error('TarBallURL for application %(appname)s not defined' % self.parameter)
      dexit(255)
    res = upload(tarballurl['Value'], self.appTar)
    if not res['OK']:
      gLogger.error("Upload to %s failed" % tarballurl['Value'], res['Message'])
      dexit(255)

  def addVersionToCS(self):
    """adds the version of the application to the CS."""
    gLogger.notice("Adding version %(appVersion)s to the CS" % self.parameter)
    existingVersions = gConfig.getSections("%(softSec)s/%(platform)s/%(appname)s" % self.parameter, [])
    if not existingVersions['OK']:
      gLogger.error("Could not find all versions available in CS: %s" % existingVersions['Message'])
      dexit(255)
    if self.appVersion in existingVersions['Value']:
      gLogger.always('Application %s %s for %s already in CS, nothing to do' % (self.appName.lower(),
                                                                                self.appVersion,
                                                                                self.platform))
      dexit(0)

    result = self.diracAdmin.csSetOption("%(softSec)s/%(platform)s/%(appname)s/%(appVersion)s/TarBall" % self.parameter,
                                         self.parameter['appTar_name'])
    if result['OK']:
      self.modifiedCS = True
    else:
      gLogger.error("Could not add version to CS")
      dexit(255)

  def addMD5SumToCS(self):
    """adds the MD5Sum of the Tarball fo the CS."""
    gLogger.notice("Adding MD5Sum to CS")
    md5sum = md5.md5(open(self.appTar).read()).hexdigest()
    result = self.diracAdmin.csSetOption("%(softSec)s/%(platform)s/%(appname)s/%(appVersion)s/Md5Sum" % self.parameter,
                                         md5sum)
    if result['OK']:
      self.modifiedCS = True
    else:
      gLogger.error("Could not add md5sum to CS")
      dexit(255)

  def addCommentToCS(self):
    """adds the comment for the TarBall to the CS."""
    gLogger.notice("Adding comment to CS: %s" % self.comment)
    result = self.diracAdmin.csSetOptionComment(
        "%(softSec)s/%(platform)s/%(appname)s/%(appVersion)s/TarBall" %
        self.parameter, self.comment)
    if not result['OK']:
      gLogger.error("Error setting comment in CS")

  def addSoftware(self):
    """run all the steps to add software to grid and CS."""

    self.checkConsistency()
    self.addVersionToCS()
    self.addMD5SumToCS()
    self.addCommentToCS()
    self.uploadTarBall()
    self.commitToCS()
    self.notifyAboutNewSoftware()


@DIRACScript()
def main():
  """uploads, registers, and sends email about new software package."""
  cliParams = _Params()
  cliParams.registerSwitches()
  Script.parseCommandLine(ignoreErrors=True)
  platform = cliParams.platform
  appName = cliParams.name
  appVersion = cliParams.version
  comment = cliParams.comment
  tarball_loc = cliParams.tarball
  if not platform or not appName or not comment:
    Script.showHelp()
    dexit(2)

  softAdder = SoftwareAdder(platform, appName, tarball_loc, appVersion, comment)
  softAdder.addSoftware()

  gLogger.notice("All done!")
  dexit(0)


if __name__ == "__main__":
  main()
